/**
 * DevExtreme (cjs/exporter/jspdf/export_data_grid_2.js)
 * Version: 21.1.0 (build 21089-0329)
 * Build date: Tue Mar 30 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.exportDataGrid = exportDataGrid;
var _type = require("../../core/utils/type");

function exportDataGrid(doc, dataGrid, options) {
    if (!(0, _type.isDefined)(options.rect)) {
        throw "options.rect is required"
    }
    var dataProvider = dataGrid.getDataProvider();
    return new Promise((function(resolve) {
        dataProvider.ready().done((function() {
            var tables = [];
            var table = {
                rect: options.rect,
                drawTableBorder: options.drawTableBorder,
                rows: []
            };
            tables.push(table);
            var columns = dataProvider.getColumns();
            var dataRowsCount = dataProvider.getRowsCount();
            for (var rowIndex = 0; rowIndex < dataRowsCount; rowIndex++) {
                if (options.onRowExporting) {
                    var drawNewTableFromThisRow = {};
                    options.onRowExporting({
                        drawNewTableFromThisRow: drawNewTableFromThisRow
                    });
                    var startNewTable = drawNewTableFromThisRow.startNewTable,
                        addPage = drawNewTableFromThisRow.addPage,
                        tableRect = drawNewTableFromThisRow.tableRect;
                    if (true === startNewTable) {
                        if (!(0, _type.isDefined)(tableRect)) {
                            throw "tableRect is required"
                        }
                        table = {
                            rect: tableRect,
                            drawTableBorder: options.drawTableBorder,
                            rows: []
                        };
                        if (true === addPage) {
                            table.drawOnNewPage = true
                        }
                        tables.push(table)
                    }
                }
                var row = [];
                table.rows.push(row);
                for (var cellIndex = 0; cellIndex < columns.length; cellIndex++) {
                    var cellData = dataProvider.getCellData(rowIndex, cellIndex, true);
                    var pdfCell = {
                        text: cellData.value
                    };
                    if (options.onCellExporting) {
                        options.onCellExporting({
                            gridCell: {
                                value: cellData.value
                            },
                            pdfCell: pdfCell
                        })
                    }
                    row.push(pdfCell);
                    if (false === pdfCell.drawLeftBorder) {
                        if (row.length > 1) {
                            row[row.length - 2].drawRightBorder = 0
                        }
                    } else if (!(0, _type.isDefined)(pdfCell.drawLeftBorder)) {
                        if (row.length > 1 && false === row[row.length - 2].drawRightBorder) {
                            pdfCell.drawLeftBorder = false
                        }
                    }
                    if (false === pdfCell.drawTopBorder) {
                        if (table.rows.length > 1) {
                            table.rows[table.rows.length - 2][row.length - 1].drawBottomBorder = false
                        }
                    } else if (!(0, _type.isDefined)(pdfCell.drawTopBorder)) {
                        if (table.rows.length > 1 && false === table.rows[table.rows.length - 2][row.length - 1].drawBottomBorder) {
                            pdfCell.drawTopBorder = false
                        }
                    }
                }
            }
            tables.forEach((function(table) {
                if (true === table.drawOnNewPage) {
                    doc.addPage()
                }
                drawTable(doc, table)
            }));
            resolve()
        }))
    }))
}

function drawTable(doc, table) {
    if (!(0, _type.isDefined)(doc)) {
        throw "doc is required"
    }

    function drawBorder(rect) {
        var drawLeftBorder = arguments.length > 1 && void 0 !== arguments[1] ? arguments[1] : true;
        var drawRightBorder = arguments.length > 2 && void 0 !== arguments[2] ? arguments[2] : true;
        var drawTopBorder = arguments.length > 3 && void 0 !== arguments[3] ? arguments[3] : true;
        var drawBottomBorder = arguments.length > 4 && void 0 !== arguments[4] ? arguments[4] : true;
        if (!(0, _type.isDefined)(rect)) {
            throw "rect is required"
        }
        var defaultBorderLineWidth = 1;
        if (!drawLeftBorder && !drawRightBorder && !drawTopBorder && !drawBottomBorder) {
            return
        } else if (drawLeftBorder && drawRightBorder && drawTopBorder && drawBottomBorder) {
            doc.setLineWidth(defaultBorderLineWidth);
            doc.rect(rect.x, rect.y, rect.w, rect.h)
        } else {
            doc.setLineWidth(defaultBorderLineWidth);
            if (drawTopBorder) {
                doc.line(rect.x, rect.y, rect.x + rect.w, rect.y)
            }
            if (drawLeftBorder) {
                doc.line(rect.x, rect.y, rect.x, rect.y + rect.h)
            }
            if (drawRightBorder) {
                doc.line(rect.x + rect.w, rect.y, rect.x + rect.w, rect.y + rect.h)
            }
            if (drawBottomBorder) {
                doc.line(rect.x, rect.y + rect.h, rect.x + rect.w, rect.y + rect.h)
            }
        }
    }

    function drawRow(rowCells) {
        if (!(0, _type.isDefined)(rowCells)) {
            throw "rowCells is required"
        }
        rowCells.forEach((function(cell) {
            if (true === cell.skip) {
                return
            }
            if (!(0, _type.isDefined)(cell.rect)) {
                throw "cell.rect is required"
            }
            if ((0, _type.isDefined)(cell.text) && "" !== cell.text) {
                var textY = cell.rect.y + cell.rect.h / 2;
                doc.text(cell.text, cell.rect.x, textY, {
                    baseline: "middle"
                })
            }
            drawBorder(cell.rect, cell.drawLeftBorder, cell.drawRightBorder, cell.drawTopBorder, cell.drawBottomBorder)
        }))
    }
    if (!(0, _type.isDefined)(table)) {
        return Promise.resolve()
    }
    if (!(0, _type.isDefined)(table.rect)) {
        throw "table.rect is required"
    }
    if ((0, _type.isDefined)(table.rows)) {
        for (var rowIndex = 0; rowIndex < table.rows.length; rowIndex++) {
            drawRow(table.rows[rowIndex])
        }
    }
    if ((0, _type.isDefined)(table.drawTableBorder) ? table.drawTableBorder : (0, _type.isDefined)(table.rows) && 0 === table.rows.length) {
        drawBorder(table.rect)
    }
}
