/**
 * DevExtreme (esm/renovation/ui/scroll_view/animated_scrollbar.js)
 * Version: 21.1.0 (build 21089-0329)
 * Build date: Tue Mar 30 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import {
    createComponentVNode,
    normalizeProps
} from "inferno";
import {
    BaseInfernoComponent
} from "devextreme-generator/modules/inferno/base_component";
import {
    isDefined
} from "../../../core/utils/type";
import devices from "../../../core/devices";
import {
    Scrollbar
} from "./scrollbar";
import {
    requestAnimationFrame,
    cancelAnimationFrame
} from "../../../animation/frame";
import {
    ScrollbarProps
} from "./scrollbar_props";
import {
    ScrollableSimulatedProps
} from "./scrollable_simulated_props";
export var OUT_BOUNDS_ACCELERATION = .5;
var realDevice = devices.real;
var isSluggishPlatform = "android" === realDevice.platform;
export var ACCELERATION = isSluggishPlatform ? .95 : .92;
export var MIN_VELOCITY_LIMIT = 1;
export var BOUNCE_MIN_VELOCITY_LIMIT = MIN_VELOCITY_LIMIT / 5;
var FRAME_DURATION = 17;
var BOUNCE_DURATION = isSluggishPlatform ? 300 : 400;
var BOUNCE_FRAMES = BOUNCE_DURATION / FRAME_DURATION;
export var BOUNCE_ACCELERATION_SUM = (1 - ACCELERATION ** BOUNCE_FRAMES) / (1 - ACCELERATION);
export var viewFunction = viewModel => {
    var {
        cancel: cancel,
        restAttributes: restAttributes,
        scrollbarRef: scrollbarRef,
        start: start
    } = viewModel, scrollbarProps = _objectWithoutPropertiesLoose(viewModel.props, ["inertiaEnabled", "onBounce"]);
    return normalizeProps(createComponentVNode(2, Scrollbar, _extends({
        onAnimatorStart: start,
        onAnimatorCancel: cancel
    }, scrollbarProps, restAttributes), null, scrollbarRef))
};
export var AnimatedScrollbarProps = _extends({}, ScrollbarProps);
var AnimatedScrollbarPropsType = {
    activeStateEnabled: AnimatedScrollbarProps.activeStateEnabled,
    expandable: AnimatedScrollbarProps.expandable,
    direction: AnimatedScrollbarProps.direction,
    scaleRatio: AnimatedScrollbarProps.scaleRatio,
    containerSize: AnimatedScrollbarProps.containerSize,
    contentSize: AnimatedScrollbarProps.contentSize,
    baseContainerSize: AnimatedScrollbarProps.baseContainerSize,
    baseContentSize: AnimatedScrollbarProps.baseContentSize,
    scrollableOffset: AnimatedScrollbarProps.scrollableOffset,
    scrollByThumb: AnimatedScrollbarProps.scrollByThumb,
    scrollScrollbar: AnimatedScrollbarProps.scrollScrollbar,
    isScrollableHovered: AnimatedScrollbarProps.isScrollableHovered,
    bounceEnabled: AnimatedScrollbarProps.bounceEnabled,
    forceVisibility: AnimatedScrollbarProps.forceVisibility,
    showScrollbar: AnimatedScrollbarProps.showScrollbar,
    scrollByContent: AnimatedScrollbarProps.scrollByContent,
    useKeyboard: AnimatedScrollbarProps.useKeyboard,
    forceGeneratePockets: AnimatedScrollbarProps.forceGeneratePockets,
    needScrollViewContentWrapper: AnimatedScrollbarProps.needScrollViewContentWrapper,
    inertiaEnabled: ScrollableSimulatedProps.inertiaEnabled,
    defaultContentTranslateOffset: ScrollableSimulatedProps.contentTranslateOffset
};
import {
    createRef as infernoCreateRef
} from "inferno";
export class AnimatedScrollbar extends BaseInfernoComponent {
    constructor(props) {
        super(props);
        this._currentState = null;
        this.scrollbarRef = infernoCreateRef();
        this.stepAnimationFrame = 0;
        this.finished = true;
        this.stopped = false;
        this.velocity = 0;
        this.animator = "inertia";
        this.bounceLocation = 0;
        this.state = {
            contentTranslateOffset: void 0 !== this.props.contentTranslateOffset ? this.props.contentTranslateOffset : this.props.defaultContentTranslateOffset
        };
        this.start = this.start.bind(this);
        this.cancel = this.cancel.bind(this);
        this.stepCore = this.stepCore.bind(this);
        this.getStepAnimationFrame = this.getStepAnimationFrame.bind(this);
        this.step = this.step.bind(this);
        this.setupBounce = this.setupBounce.bind(this);
        this.complete = this.complete.bind(this);
        this.stop = this.stop.bind(this);
        this.suppressInertia = this.suppressInertia.bind(this);
        this.crossBoundOnNextStep = this.crossBoundOnNextStep.bind(this);
        this.inBounds = this.inBounds.bind(this);
        this.getMaxOffset = this.getMaxOffset.bind(this);
        this.scrollStep = this.scrollStep.bind(this);
        this.move = this.move.bind(this);
        this.stopComplete = this.stopComplete.bind(this);
        this.scrollComplete = this.scrollComplete.bind(this);
        this.getLocation = this.getLocation.bind(this);
        this.boundLocation = this.boundLocation.bind(this);
        this.getScrollLocation = this.getScrollLocation.bind(this);
        this.getMinOffset = this.getMinOffset.bind(this);
        this.validateEvent = this.validateEvent.bind(this);
        this.isThumb = this.isThumb.bind(this);
        this.reachedMin = this.reachedMin.bind(this);
        this.reachedMax = this.reachedMax.bind(this);
        this.initHandler = this.initHandler.bind(this);
        this.startHandler = this.startHandler.bind(this);
        this.moveHandler = this.moveHandler.bind(this);
        this.endHandler = this.endHandler.bind(this);
        this.stopHandler = this.stopHandler.bind(this);
        this.scrollByHandler = this.scrollByHandler.bind(this)
    }
    get __state_contentTranslateOffset() {
        var state = this._currentState || this.state;
        return void 0 !== this.props.contentTranslateOffset ? this.props.contentTranslateOffset : state.contentTranslateOffset
    }
    set_contentTranslateOffset(value) {
        this.setState(state => {
            var _this$props$contentTr, _this$props;
            this._currentState = state;
            var newValue = value();
            null === (_this$props$contentTr = (_this$props = this.props).contentTranslateOffsetChange) || void 0 === _this$props$contentTr ? void 0 : _this$props$contentTr.call(_this$props, newValue);
            this._currentState = null;
            return {
                contentTranslateOffset: newValue
            }
        })
    }
    start(animatorName, receivedVelocity, thumbScrolling, crossThumbScrolling) {
        this.animator = animatorName;
        if (this.isBounceAnimator) {
            var _this$props$onBounce, _this$props2;
            null === (_this$props$onBounce = (_this$props2 = this.props).onBounce) || void 0 === _this$props$onBounce ? void 0 : _this$props$onBounce.call(_this$props2);
            this.setupBounce()
        } else {
            if (!thumbScrolling && crossThumbScrolling) {
                this.velocity = 0
            } else {
                this.velocity = receivedVelocity || 0
            }
            this.suppressInertia(thumbScrolling)
        }
        this.stopped = false;
        this.finished = false;
        this.stepCore()
    }
    cancel() {
        this.stopped = true;
        cancelAnimationFrame(this.stepAnimationFrame)
    }
    stepCore() {
        if (this.stopped) {
            this.stop();
            return
        }
        if (this.isFinished) {
            this.finished = true;
            this.complete();
            return
        }
        this.step();
        this.stepAnimationFrame = this.getStepAnimationFrame()
    }
    getStepAnimationFrame() {
        return requestAnimationFrame(this.stepCore)
    }
    step() {
        if (!this.props.bounceEnabled && !this.inBounds()) {
            this.velocity = 0
        }
        this.scrollStep(this.velocity);
        this.velocity *= this.acceleration
    }
    setupBounce() {
        var bounceDistance = this.boundLocation() - this.getLocation();
        this.velocity = bounceDistance / BOUNCE_ACCELERATION_SUM
    }
    complete() {
        if (this.isBounceAnimator) {
            this.move(this.boundLocation())
        }
        this.scrollComplete()
    }
    get isBounceAnimator() {
        return "bounce" === this.animator
    }
    get isFinished() {
        if (this.isBounceAnimator) {
            return this.crossBoundOnNextStep() || Math.abs(this.velocity) <= BOUNCE_MIN_VELOCITY_LIMIT
        }
        return Math.abs(this.velocity) <= MIN_VELOCITY_LIMIT
    }
    get inProgress() {
        return !(this.stopped || this.finished)
    }
    get acceleration() {
        var _this$scrollbarRef;
        if (!isDefined(null === (_this$scrollbarRef = this.scrollbarRef) || void 0 === _this$scrollbarRef ? void 0 : _this$scrollbarRef.current)) {
            return 0
        }
        return this.inBounds() || this.isBounceAnimator ? ACCELERATION : OUT_BOUNDS_ACCELERATION
    }
    stop() {
        this.stopComplete()
    }
    suppressInertia(thumbScrolling) {
        if (!this.props.inertiaEnabled || thumbScrolling) {
            this.velocity = 0
        }
    }
    crossBoundOnNextStep() {
        var location = this.getLocation();
        var nextLocation = location + this.velocity;
        var minOffset = this.getMinOffset();
        return location < minOffset && nextLocation >= minOffset || location > this.getMaxOffset() && nextLocation <= this.getMaxOffset()
    }
    inBounds() {
        var scrollbar = this.scrollbarRef.current;
        if (!isDefined(scrollbar)) {
            return false
        }
        return scrollbar.inBounds()
    }
    getMaxOffset() {
        return this.scrollbarRef.current.getMaxOffset()
    }
    scrollStep(delta) {
        return this.scrollbarRef.current.scrollStep(delta)
    }
    move(location) {
        return this.scrollbarRef.current.move(location)
    }
    stopComplete() {
        return this.scrollbarRef.current.stopComplete()
    }
    scrollComplete() {
        return this.scrollbarRef.current.scrollComplete()
    }
    getLocation() {
        return this.scrollbarRef.current.getLocation()
    }
    get restAttributes() {
        var _this$props$contentTr2 = _extends({}, this.props, {
                contentTranslateOffset: this.__state_contentTranslateOffset
            }),
            restProps = _objectWithoutPropertiesLoose(_this$props$contentTr2, ["activeStateEnabled", "baseContainerSize", "baseContentSize", "bounceEnabled", "children", "classes", "containerSize", "contentPositionChange", "contentSize", "contentTranslateOffset", "contentTranslateOffsetChange", "defaultContentTranslateOffset", "direction", "expandable", "forceGeneratePockets", "forceVisibility", "hoverStateEnabled", "inertiaEnabled", "isScrollableHovered", "needScrollViewContentWrapper", "onAnimatorCancel", "onAnimatorStart", "onBounce", "onScroll", "onUpdated", "scaleRatio", "scrollByContent", "scrollByThumb", "scrollScrollbar", "scrollableOffset", "showScrollbar", "useKeyboard"]);
        return restProps
    }
    boundLocation(value) {
        return this.scrollbarRef.current.boundLocation(value)
    }
    getScrollLocation() {
        return this.scrollbarRef.current.getScrollLocation()
    }
    getMinOffset() {
        return this.scrollbarRef.current.getMinOffset()
    }
    validateEvent(e) {
        return this.scrollbarRef.current.validateEvent(e)
    }
    isThumb(element) {
        return this.scrollbarRef.current.isThumb(element)
    }
    reachedMin() {
        return this.scrollbarRef.current.reachedMin()
    }
    reachedMax() {
        return this.scrollbarRef.current.reachedMax()
    }
    initHandler(e, crossThumbScrolling) {
        this.scrollbarRef.current.initHandler(e, crossThumbScrolling)
    }
    startHandler() {
        this.scrollbarRef.current.startHandler()
    }
    moveHandler(delta) {
        this.scrollbarRef.current.moveHandler(delta)
    }
    endHandler(velocity) {
        this.scrollbarRef.current.endHandler(velocity)
    }
    stopHandler() {
        this.scrollbarRef.current.stopHandler()
    }
    scrollByHandler(delta) {
        this.scrollbarRef.current.scrollByHandler(delta)
    }
    render() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props, {
                contentTranslateOffset: this.__state_contentTranslateOffset
            }),
            scrollbarRef: this.scrollbarRef,
            start: this.start,
            cancel: this.cancel,
            stepCore: this.stepCore,
            getStepAnimationFrame: this.getStepAnimationFrame,
            step: this.step,
            setupBounce: this.setupBounce,
            complete: this.complete,
            isBounceAnimator: this.isBounceAnimator,
            isFinished: this.isFinished,
            inProgress: this.inProgress,
            acceleration: this.acceleration,
            stop: this.stop,
            suppressInertia: this.suppressInertia,
            crossBoundOnNextStep: this.crossBoundOnNextStep,
            inBounds: this.inBounds,
            getMaxOffset: this.getMaxOffset,
            scrollStep: this.scrollStep,
            move: this.move,
            stopComplete: this.stopComplete,
            scrollComplete: this.scrollComplete,
            getLocation: this.getLocation,
            restAttributes: this.restAttributes
        })
    }
}
AnimatedScrollbar.defaultProps = _extends({}, AnimatedScrollbarPropsType);
