/**
 * DevExtreme (esm/renovation/ui/scroll_view/scrollbar.js)
 * Version: 21.1.0 (build 21089-0329)
 * Build date: Tue Mar 30 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import {
    createVNode,
    createComponentVNode,
    normalizeProps
} from "inferno";
import {
    InfernoEffect
} from "devextreme-generator/modules/inferno/effect";
import {
    InfernoComponent
} from "devextreme-generator/modules/inferno/base_component";
import {
    Widget
} from "../common/widget";
import {
    combineClasses
} from "../../utils/combine_classes";
import domAdapter from "../../../core/dom_adapter";
import {
    isDefined
} from "../../../core/utils/type";
import {
    isDxMouseWheelEvent
} from "../../../events/utils/index";
import {
    ScrollbarProps
} from "./scrollbar_props";
import {
    DIRECTION_HORIZONTAL,
    SCROLLABLE_SCROLLBAR_CLASS
} from "./scrollable_utils";
import {
    dxPointerDown,
    dxPointerUp
} from "../../../events/short";
import {
    ScrollableSimulatedProps
} from "./scrollable_simulated_props";
var OUT_BOUNDS_ACCELERATION = .5;
var SCROLLABLE_SCROLLBAR_ACTIVE_CLASS = "dx-scrollable-scrollbar-active";
var SCROLLABLE_SCROLL_CLASS = "dx-scrollable-scroll";
var SCROLLABLE_SCROLL_CONTENT_CLASS = "dx-scrollable-scroll-content";
var HOVER_ENABLED_STATE = "dx-scrollbar-hoverable";
var MAX_OFFSET = 0;
var THUMB_MIN_SIZE = 15;
var HIDE_SCROLLBAR_TIMEOUT = 500;
export var viewFunction = viewModel => {
    var {
        cssClasses: cssClasses,
        hoverStateEnabled: hoverStateEnabled,
        isVisible: isVisible,
        onHoverEndHandler: onHoverEndHandler,
        onHoverStartHandler: onHoverStartHandler,
        props: {
            activeStateEnabled: activeStateEnabled
        },
        restAttributes: restAttributes,
        scrollRef: scrollRef,
        scrollbarRef: scrollbarRef,
        styles: styles
    } = viewModel;
    return normalizeProps(createComponentVNode(2, Widget, _extends({
        rootElementRef: scrollbarRef,
        classes: cssClasses,
        activeStateEnabled: activeStateEnabled,
        hoverStateEnabled: hoverStateEnabled,
        visible: isVisible,
        onHoverStart: onHoverStartHandler,
        onHoverEnd: onHoverEndHandler
    }, restAttributes, {
        children: createVNode(1, "div", viewModel.scrollClasses, createVNode(1, "div", SCROLLABLE_SCROLL_CONTENT_CLASS), 2, {
            style: styles
        }, null, scrollRef)
    })))
};
export var ScrollbarPropsType = {
    activeStateEnabled: ScrollbarProps.activeStateEnabled,
    expandable: ScrollbarProps.expandable,
    direction: ScrollbarProps.direction,
    scaleRatio: ScrollbarProps.scaleRatio,
    containerSize: ScrollbarProps.containerSize,
    contentSize: ScrollbarProps.contentSize,
    baseContainerSize: ScrollbarProps.baseContainerSize,
    baseContentSize: ScrollbarProps.baseContentSize,
    scrollableOffset: ScrollbarProps.scrollableOffset,
    scrollByThumb: ScrollbarProps.scrollByThumb,
    scrollScrollbar: ScrollbarProps.scrollScrollbar,
    isScrollableHovered: ScrollbarProps.isScrollableHovered,
    bounceEnabled: ScrollbarProps.bounceEnabled,
    forceVisibility: ScrollbarProps.forceVisibility,
    showScrollbar: ScrollbarProps.showScrollbar,
    scrollByContent: ScrollbarProps.scrollByContent,
    useKeyboard: ScrollbarProps.useKeyboard,
    forceGeneratePockets: ScrollbarProps.forceGeneratePockets,
    needScrollViewContentWrapper: ScrollbarProps.needScrollViewContentWrapper,
    defaultContentTranslateOffset: ScrollableSimulatedProps.contentTranslateOffset
};
import {
    createRef as infernoCreateRef
} from "inferno";
export class Scrollbar extends InfernoComponent {
    constructor(props) {
        super(props);
        this._currentState = null;
        this.location = 0;
        this.thumbScrolling = false;
        this.crossThumbScrolling = false;
        this.prevThumbRatio = 1;
        this.scrollbarRef = infernoCreateRef();
        this.scrollRef = infernoCreateRef();
        this.state = {
            showOnScrollByWheel: void 0,
            hovered: false,
            active: false,
            visibility: false,
            scrollLocation: 0,
            contentTranslateOffset: void 0 !== this.props.contentTranslateOffset ? this.props.contentTranslateOffset : this.props.defaultContentTranslateOffset
        };
        this.pointerDownEffect = this.pointerDownEffect.bind(this);
        this.pointerUpEffect = this.pointerUpEffect.bind(this);
        this.isThumb = this.isThumb.bind(this);
        this.isScrollbar = this.isScrollbar.bind(this);
        this.validateEvent = this.validateEvent.bind(this);
        this.reachedMin = this.reachedMin.bind(this);
        this.reachedMax = this.reachedMax.bind(this);
        this.getLocation = this.getLocation.bind(this);
        this.getScrollLocation = this.getScrollLocation.bind(this);
        this.setLocation = this.setLocation.bind(this);
        this.inBounds = this.inBounds.bind(this);
        this.boundLocation = this.boundLocation.bind(this);
        this.getMinOffset = this.getMinOffset.bind(this);
        this.getMaxOffset = this.getMaxOffset.bind(this);
        this.initHandler = this.initHandler.bind(this);
        this.startHandler = this.startHandler.bind(this);
        this.moveHandler = this.moveHandler.bind(this);
        this.hide = this.hide.bind(this);
        this.endHandler = this.endHandler.bind(this);
        this.stopHandler = this.stopHandler.bind(this);
        this.scrollByHandler = this.scrollByHandler.bind(this);
        this.scrollComplete = this.scrollComplete.bind(this);
        this.scrollToBounds = this.scrollToBounds.bind(this);
        this.stopComplete = this.stopComplete.bind(this);
        this.resetThumbScrolling = this.resetThumbScrolling.bind(this);
        this.scrollBy = this.scrollBy.bind(this);
        this.stopScrolling = this.stopScrolling.bind(this);
        this.prepareThumbScrolling = this.prepareThumbScrolling.bind(this);
        this.moveToMouseLocation = this.moveToMouseLocation.bind(this);
        this.scrollStep = this.scrollStep.bind(this);
        this.suppressBounce = this.suppressBounce.bind(this);
        this.move = this.move.bind(this);
        this.moveScrollbar = this.moveScrollbar.bind(this);
        this.moveContent = this.moveContent.bind(this);
        this.containerToContentRatio = this.containerToContentRatio.bind(this);
        this.baseContainerToContentRatio = this.baseContainerToContentRatio.bind(this);
        this.feedbackOn = this.feedbackOn.bind(this);
        this.feedbackOff = this.feedbackOff.bind(this);
        this.onHoverStartHandler = this.onHoverStartHandler.bind(this);
        this.onHoverEndHandler = this.onHoverEndHandler.bind(this)
    }
    createEffects() {
        return [new InfernoEffect(this.pointerDownEffect, []), new InfernoEffect(this.pointerUpEffect, [])]
    }
    updateEffects() {
        var _this$_effects$, _this$_effects$2;
        null === (_this$_effects$ = this._effects[0]) || void 0 === _this$_effects$ ? void 0 : _this$_effects$.update([]);
        null === (_this$_effects$2 = this._effects[1]) || void 0 === _this$_effects$2 ? void 0 : _this$_effects$2.update([])
    }
    get showOnScrollByWheel() {
        var state = this._currentState || this.state;
        return state.showOnScrollByWheel
    }
    set_showOnScrollByWheel(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                showOnScrollByWheel: newValue
            }
        })
    }
    get hovered() {
        var state = this._currentState || this.state;
        return state.hovered
    }
    set_hovered(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                hovered: newValue
            }
        })
    }
    get active() {
        var state = this._currentState || this.state;
        return state.active
    }
    set_active(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                active: newValue
            }
        })
    }
    get visibility() {
        var state = this._currentState || this.state;
        return state.visibility
    }
    set_visibility(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                visibility: newValue
            }
        })
    }
    get scrollLocation() {
        var state = this._currentState || this.state;
        return state.scrollLocation
    }
    set_scrollLocation(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                scrollLocation: newValue
            }
        })
    }
    get __state_contentTranslateOffset() {
        var state = this._currentState || this.state;
        return void 0 !== this.props.contentTranslateOffset ? this.props.contentTranslateOffset : state.contentTranslateOffset
    }
    set_contentTranslateOffset(value) {
        this.setState(state => {
            var _this$props$contentTr, _this$props;
            this._currentState = state;
            var newValue = value();
            null === (_this$props$contentTr = (_this$props = this.props).contentTranslateOffsetChange) || void 0 === _this$props$contentTr ? void 0 : _this$props$contentTr.call(_this$props, newValue);
            this._currentState = null;
            return {
                contentTranslateOffset: newValue
            }
        })
    }
    pointerDownEffect() {
        dxPointerDown.on(this.scrollRef.current, () => {
            this.feedbackOn()
        }, {
            namespace: "dxScrollbar"
        });
        return () => dxPointerDown.off(this.scrollRef.current, {
            namespace: "dxScrollbar"
        })
    }
    pointerUpEffect() {
        dxPointerUp.on(domAdapter.getDocument(), () => {
            this.feedbackOff()
        }, {
            namespace: "dxScrollbar"
        });
        return () => dxPointerUp.off(this.scrollRef.current, {
            namespace: "dxScrollbar"
        })
    }
    get scrollTranslateOffset() {
        return _extends({}, {
            left: 0,
            top: 0
        }, {
            [this.scrollProp]: this.scrollPosition
        })
    }
    get scrollPosition() {
        var thumbRatioChanged = Math.abs(this.thumbRatio - this.prevThumbRatio) > .01;
        if (thumbRatioChanged) {
            this.set_scrollLocation(() => this.boundLocation(this.scrollLocation));
            this.prevThumbRatio = this.thumbRatio
        }
        return -this.scrollLocation * this.thumbRatio
    }
    get minOffset() {
        return Math.round(-Math.max(this.props.contentSize - this.props.containerSize, 0))
    }
    get maxOffset() {
        return MAX_OFFSET
    }
    get axis() {
        return this.props.direction === DIRECTION_HORIZONTAL ? "x" : "y"
    }
    get scrollProp() {
        return this.props.direction === DIRECTION_HORIZONTAL ? "left" : "top"
    }
    get fullScrollProp() {
        return this.props.direction === DIRECTION_HORIZONTAL ? "scrollLeft" : "scrollTop"
    }
    hide() {
        this.set_visibility(() => false);
        if (isDefined(this.showOnScrollByWheel) && "onScroll" === this.props.showScrollbar) {
            setTimeout(() => {
                this.set_showOnScrollByWheel(() => {})
            }, HIDE_SCROLLBAR_TIMEOUT)
        }
    }
    scrollToBounds() {
        var _this$props$onAnimato, _this$props2;
        if (this.inBounds()) {
            return
        }
        null === (_this$props$onAnimato = (_this$props2 = this.props).onAnimatorStart) || void 0 === _this$props$onAnimato ? void 0 : _this$props$onAnimato.call(_this$props2, "bounce")
    }
    resetThumbScrolling() {
        this.thumbScrolling = false;
        this.crossThumbScrolling = false
    }
    scrollBy(delta) {
        var distance = delta[this.axis];
        if (!this.inBounds()) {
            distance *= OUT_BOUNDS_ACCELERATION
        }
        this.scrollStep(distance)
    }
    stopScrolling() {
        var _this$props$onAnimato2, _this$props3;
        this.hide();
        null === (_this$props$onAnimato2 = (_this$props3 = this.props).onAnimatorCancel) || void 0 === _this$props$onAnimato2 ? void 0 : _this$props$onAnimato2.call(_this$props3)
    }
    prepareThumbScrolling(e, currentCrossThumbScrolling) {
        if (isDxMouseWheelEvent(e.originalEvent)) {
            if ("onScroll" === this.props.showScrollbar) {
                this.set_showOnScrollByWheel(() => true)
            }
            return
        }
        var {
            target: target
        } = e.originalEvent;
        var {
            scrollByThumb: scrollByThumb
        } = this.props;
        var scrollbarClicked = scrollByThumb && this.isScrollbar(target);
        if (scrollbarClicked) {
            this.moveToMouseLocation(e)
        }
        var currentThumbScrolling = scrollbarClicked || scrollByThumb && this.isThumb(target);
        this.thumbScrolling = currentThumbScrolling;
        this.crossThumbScrolling = !currentThumbScrolling && currentCrossThumbScrolling;
        if (currentThumbScrolling) {
            this.feedbackOn()
        }
    }
    moveToMouseLocation(e) {
        var mouseLocation = e["page".concat(this.axis.toUpperCase())] - this.props.scrollableOffset;
        var location = this.getLocation() + mouseLocation / this.containerToContentRatio() - this.props.containerSize / 2;
        this.scrollStep(-Math.round(location))
    }
    suppressBounce() {
        if (this.props.bounceEnabled || this.inBounds()) {
            return
        }
        this.setLocation(this.boundLocation())
    }
    moveContent() {
        var _this$props$contentPo, _this$props4, _this$props$contentTr2, _this$props5;
        var location = this.getLocation();
        var currentTranslateOffset;
        if (location > 0) {
            currentTranslateOffset = location
        } else if (location <= this.minOffset) {
            currentTranslateOffset = location - this.minOffset
        } else {
            currentTranslateOffset = location % 1
        }
        null === (_this$props$contentPo = (_this$props4 = this.props).contentPositionChange) || void 0 === _this$props$contentPo ? void 0 : _this$props$contentPo.call(_this$props4, this.fullScrollProp, location, this.props.scaleRatio);
        null === (_this$props$contentTr2 = (_this$props5 = this.props).contentTranslateOffsetChange) || void 0 === _this$props$contentTr2 ? void 0 : _this$props$contentTr2.call(_this$props5, {
            [this.scrollProp]: currentTranslateOffset
        })
    }
    get thumbSize() {
        var {
            containerSize: containerSize,
            scaleRatio: scaleRatio
        } = this.props;
        var size = Math.round(Math.max(Math.round(containerSize * this.containerToContentRatio()), THUMB_MIN_SIZE));
        return size / scaleRatio
    }
    get thumbRatio() {
        var {
            containerSize: containerSize,
            contentSize: contentSize,
            scaleRatio: scaleRatio
        } = this.props;
        if (contentSize) {
            return (containerSize - this.thumbSize) / (scaleRatio * (contentSize - containerSize))
        }
        return 1
    }
    containerToContentRatio() {
        var {
            containerSize: containerSize,
            contentSize: contentSize
        } = this.props;
        return contentSize ? containerSize / contentSize : containerSize
    }
    baseContainerToContentRatio() {
        var {
            baseContainerSize: baseContainerSize,
            baseContentSize: baseContentSize
        } = this.props;
        return baseContentSize ? baseContainerSize / baseContentSize : baseContainerSize
    }
    get dimension() {
        return this.props.direction === DIRECTION_HORIZONTAL ? "width" : "height"
    }
    feedbackOn() {
        this.set_active(() => true)
    }
    feedbackOff() {
        this.set_active(() => false)
    }
    onHoverStartHandler() {
        if ("onHover" === this.props.showScrollbar) {
            this.set_hovered(() => true)
        }
    }
    onHoverEndHandler() {
        if ("onHover" === this.props.showScrollbar) {
            this.set_hovered(() => false)
        }
    }
    get cssClasses() {
        var {
            direction: direction
        } = this.props;
        var classesMap = {
            [SCROLLABLE_SCROLLBAR_CLASS]: true,
            ["dx-scrollbar-".concat(direction)]: true,
            [SCROLLABLE_SCROLLBAR_ACTIVE_CLASS]: !!this.active,
            [HOVER_ENABLED_STATE]: !!this.hoverStateEnabled
        };
        return combineClasses(classesMap)
    }
    get styles() {
        return {
            [this.dimension]: this.thumbSize || THUMB_MIN_SIZE,
            transform: "never" === this.props.showScrollbar ? "none" : "translate(".concat(this.scrollTranslateOffset.left, "px, ").concat(this.scrollTranslateOffset.top, "px)")
        }
    }
    get scrollClasses() {
        return combineClasses({
            [SCROLLABLE_SCROLL_CLASS]: true,
            "dx-state-invisible": !this.visible
        })
    }
    get isVisible() {
        return "never" !== this.props.showScrollbar && this.baseContainerToContentRatio() < 1
    }
    get visible() {
        var {
            forceVisibility: forceVisibility,
            showScrollbar: showScrollbar
        } = this.props;
        if (!this.isVisible) {
            return false
        }
        if ("onHover" === showScrollbar) {
            return this.visibility || this.props.isScrollableHovered || this.hovered
        }
        if ("always" === showScrollbar) {
            return true
        }
        return forceVisibility || this.visibility || !!this.showOnScrollByWheel
    }
    get hoverStateEnabled() {
        var {
            scrollByThumb: scrollByThumb,
            showScrollbar: showScrollbar
        } = this.props;
        return ("onHover" === showScrollbar || "always" === showScrollbar) && scrollByThumb
    }
    get restAttributes() {
        var _this$props$contentTr3 = _extends({}, this.props, {
                contentTranslateOffset: this.__state_contentTranslateOffset
            }),
            restProps = _objectWithoutPropertiesLoose(_this$props$contentTr3, ["activeStateEnabled", "baseContainerSize", "baseContentSize", "bounceEnabled", "children", "classes", "containerSize", "contentPositionChange", "contentSize", "contentTranslateOffset", "contentTranslateOffsetChange", "defaultContentTranslateOffset", "direction", "expandable", "forceGeneratePockets", "forceVisibility", "hoverStateEnabled", "isScrollableHovered", "needScrollViewContentWrapper", "onAnimatorCancel", "onAnimatorStart", "onScroll", "onUpdated", "scaleRatio", "scrollByContent", "scrollByThumb", "scrollScrollbar", "scrollableOffset", "showScrollbar", "useKeyboard"]);
        return restProps
    }
    isThumb(element) {
        var _this$scrollbarRef$cu, _this$scrollbarRef$cu2;
        return (null === (_this$scrollbarRef$cu = this.scrollbarRef.current) || void 0 === _this$scrollbarRef$cu ? void 0 : _this$scrollbarRef$cu.querySelector(".".concat(SCROLLABLE_SCROLL_CLASS))) === element || (null === (_this$scrollbarRef$cu2 = this.scrollbarRef.current) || void 0 === _this$scrollbarRef$cu2 ? void 0 : _this$scrollbarRef$cu2.querySelector(".".concat(SCROLLABLE_SCROLL_CONTENT_CLASS))) === element
    }
    isScrollbar(element) {
        return element === this.scrollbarRef.current
    }
    validateEvent(e) {
        var {
            target: target
        } = e.originalEvent;
        return this.isThumb(target) || this.isScrollbar(target)
    }
    reachedMin() {
        return this.getLocation() <= this.minOffset
    }
    reachedMax() {
        return this.getLocation() >= this.maxOffset
    }
    getLocation() {
        return this.location
    }
    getScrollLocation() {
        return this.scrollLocation
    }
    setLocation(value) {
        this.location = value
    }
    inBounds() {
        return this.boundLocation() === this.getLocation()
    }
    boundLocation(value) {
        var currentLocation = isDefined(value) ? value : this.getLocation();
        return Math.max(Math.min(currentLocation, this.maxOffset), this.minOffset)
    }
    getMinOffset() {
        return this.minOffset
    }
    getMaxOffset() {
        return this.maxOffset
    }
    initHandler(e, crossThumbScrolling) {
        this.stopScrolling();
        this.prepareThumbScrolling(e, crossThumbScrolling)
    }
    startHandler() {
        this.set_visibility(() => true)
    }
    moveHandler(delta) {
        if (this.crossThumbScrolling) {
            return
        }
        var distance = delta;
        if (this.thumbScrolling) {
            distance[this.axis] = -Math.round(distance[this.axis] / this.containerToContentRatio())
        }
        this.scrollBy(distance)
    }
    endHandler(velocity) {
        var _this$props$onAnimato3, _this$props6;
        null === (_this$props$onAnimato3 = (_this$props6 = this.props).onAnimatorStart) || void 0 === _this$props$onAnimato3 ? void 0 : _this$props$onAnimato3.call(_this$props6, "inertia", velocity[this.axis], this.thumbScrolling, this.crossThumbScrolling);
        this.resetThumbScrolling()
    }
    stopHandler() {
        this.hide();
        if (this.thumbScrolling) {
            this.scrollComplete()
        } else {
            this.scrollToBounds()
        }
        this.resetThumbScrolling()
    }
    scrollByHandler(delta) {
        this.scrollBy(delta);
        this.scrollComplete()
    }
    scrollComplete() {
        if (this.inBounds()) {
            this.hide()
        }
        this.scrollToBounds()
    }
    stopComplete() {}
    scrollStep(delta) {
        this.setLocation(this.getLocation() + delta);
        this.suppressBounce();
        this.move()
    }
    move(location) {
        this.moveScrollbar(location);
        this.moveContent()
    }
    moveScrollbar(location) {
        var currentLocation = void 0 !== location ? location * this.props.scaleRatio : this.getLocation();
        this.setLocation(currentLocation);
        this.set_scrollLocation(() => currentLocation)
    }
    render() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props, {
                contentTranslateOffset: this.__state_contentTranslateOffset
            }),
            showOnScrollByWheel: this.showOnScrollByWheel,
            hovered: this.hovered,
            active: this.active,
            visibility: this.visibility,
            scrollLocation: this.scrollLocation,
            scrollbarRef: this.scrollbarRef,
            scrollRef: this.scrollRef,
            scrollTranslateOffset: this.scrollTranslateOffset,
            scrollPosition: this.scrollPosition,
            minOffset: this.minOffset,
            maxOffset: this.maxOffset,
            axis: this.axis,
            scrollProp: this.scrollProp,
            fullScrollProp: this.fullScrollProp,
            hide: this.hide,
            scrollToBounds: this.scrollToBounds,
            resetThumbScrolling: this.resetThumbScrolling,
            scrollBy: this.scrollBy,
            stopScrolling: this.stopScrolling,
            prepareThumbScrolling: this.prepareThumbScrolling,
            moveToMouseLocation: this.moveToMouseLocation,
            suppressBounce: this.suppressBounce,
            moveContent: this.moveContent,
            thumbSize: this.thumbSize,
            thumbRatio: this.thumbRatio,
            containerToContentRatio: this.containerToContentRatio,
            baseContainerToContentRatio: this.baseContainerToContentRatio,
            dimension: this.dimension,
            feedbackOn: this.feedbackOn,
            feedbackOff: this.feedbackOff,
            onHoverStartHandler: this.onHoverStartHandler,
            onHoverEndHandler: this.onHoverEndHandler,
            cssClasses: this.cssClasses,
            styles: this.styles,
            scrollClasses: this.scrollClasses,
            isVisible: this.isVisible,
            visible: this.visible,
            hoverStateEnabled: this.hoverStateEnabled,
            restAttributes: this.restAttributes
        })
    }
}
Scrollbar.defaultProps = _extends({}, ScrollbarPropsType);
