/**
 * DevExtreme (esm/renovation/ui/scroll_view/top_pocket.js)
 * Version: 21.1.0 (build 21089-0329)
 * Build date: Tue Mar 30 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import {
    createVNode,
    createFragment
} from "inferno";
import {
    Fragment
} from "inferno";
import {
    BaseInfernoComponent
} from "devextreme-generator/modules/inferno/base_component";
import devices from "../../../core/devices";
import {
    isDefined
} from "../../../core/utils/type";
import messageLocalization from "../../../localization/message";
import {
    TopPocketProps,
    TopPocketState
} from "./top_pocket_props";
var SCROLLVIEW_TOP_POCKET_CLASS = "dx-scrollview-top-pocket";
var SCROLLVIEW_PULLDOWN = "dx-scrollview-pull-down";
var SCROLLVIEW_PULLDOWN_IMAGE_CLASS = "dx-scrollview-pull-down-image";
var SCROLLVIEW_PULLDOWN_INDICATOR_CLASS = "dx-scrollview-pull-down-indicator";
var SCROLLVIEW_PULLDOWN_TEXT_CLASS = "dx-scrollview-pull-down-text";
var SCROLLVIEW_PULLDOWN_VISIBLE_TEXT_CLASS = "dx-scrollview-pull-down-text-visible";

function getVisibleClass(currentState, expectedState) {
    return currentState === expectedState ? SCROLLVIEW_PULLDOWN_VISIBLE_TEXT_CLASS : void 0
}
export var viewFunction = viewModel => {
    var {
        pocketState: pocketState,
        pulledDownText: pulledDownText,
        pullingDownText: pullingDownText,
        refreshStrategy: refreshStrategy,
        refreshingText: refreshingText
    } = viewModel;
    return createVNode(1, "div", SCROLLVIEW_TOP_POCKET_CLASS, createVNode(1, "div", SCROLLVIEW_PULLDOWN, [createVNode(1, "div", SCROLLVIEW_PULLDOWN_IMAGE_CLASS), createVNode(1, "div", SCROLLVIEW_PULLDOWN_INDICATOR_CLASS), createVNode(1, "div", SCROLLVIEW_PULLDOWN_TEXT_CLASS, "swipeDown" !== refreshStrategy && createFragment([createVNode(1, "div", getVisibleClass(pocketState, TopPocketState.STATE_RELEASED), pullingDownText, 0), createVNode(1, "div", getVisibleClass(pocketState, TopPocketState.STATE_READY), pulledDownText, 0), createVNode(1, "div", getVisibleClass(pocketState, TopPocketState.STATE_REFRESHING), refreshingText, 0)], 4), 0)], 4), 2)
};
export class TopPocket extends BaseInfernoComponent {
    constructor(props) {
        super(props);
        this._currentState = null;
        this.state = {
            pocketState: TopPocketState.STATE_RELEASED
        }
    }
    get pocketState() {
        var state = this._currentState || this.state;
        return state.pocketState
    }
    set_pocketState(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                pocketState: newValue
            }
        })
    }
    get refreshStrategy() {
        return this.props.refreshStrategy || ("android" === devices.real().platform ? "swipeDown" : "pullDown")
    }
    get pullingDownText() {
        var {
            pullingDownText: pullingDownText
        } = this.props;
        if (isDefined(pullingDownText)) {
            return pullingDownText
        }
        return messageLocalization.format("dxScrollView-pullingDownText")
    }
    get pulledDownText() {
        var {
            pulledDownText: pulledDownText
        } = this.props;
        if (isDefined(pulledDownText)) {
            return pulledDownText
        }
        return messageLocalization.format("dxScrollView-pulledDownText")
    }
    get refreshingText() {
        var {
            refreshingText: refreshingText
        } = this.props;
        if (isDefined(refreshingText)) {
            return refreshingText
        }
        return messageLocalization.format("dxScrollView-refreshingText")
    }
    get restAttributes() {
        var _this$props = this.props,
            restProps = _objectWithoutPropertiesLoose(_this$props, ["pulledDownText", "pullingDownText", "refreshStrategy", "refreshingText"]);
        return restProps
    }
    render() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props),
            pocketState: this.pocketState,
            refreshStrategy: this.refreshStrategy,
            pullingDownText: this.pullingDownText,
            pulledDownText: this.pulledDownText,
            refreshingText: this.refreshingText,
            restAttributes: this.restAttributes
        })
    }
}
TopPocket.defaultProps = _extends({}, TopPocketProps);
