/**
* DevExtreme (file_management/provider_base.d.ts)
* Version: 21.1.0 (build 21089-0329)
* Build date: Tue Mar 30 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import FileSystemItem from './file_system_item';
import UploadInfo from './upload_info';

import {
    TPromise
} from '../core/utils/deferred';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface FileSystemProviderBaseOptions<T = FileSystemProviderBase> {
    /**
     */
    dateModifiedExpr?: string | Function;
    /**
     */
    isDirectoryExpr?: string | Function;
    /**
     */
    keyExpr?: string | Function;
    /**
     */
    nameExpr?: string | Function;
    /**
     */
    sizeExpr?: string | Function;
    /**
     */
    thumbnailExpr?: string | Function;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export default class FileSystemProviderBase {
    constructor(options?: FileSystemProviderBaseOptions)
    /**
     */
    getItems(parentDirectory: FileSystemItem): TPromise<Array<FileSystemItem>>;

    /**
     */
    renameItem(item: FileSystemItem, newName: string): TPromise<any>;

    /**
     */
    createDirectory(parentDirectory: FileSystemItem, name: string): TPromise<any>;

    /**
     */
    deleteItems(items: Array<FileSystemItem>): Array<TPromise<any>>;

    /**
     */
    moveItems(items: Array<FileSystemItem>, destinationDirectory: FileSystemItem): Array<TPromise<any>>;

    /**
     */
    copyItems(items: Array<FileSystemItem>, destinationDirectory: FileSystemItem): Array<TPromise<any>>;

    /**
     */
    uploadFileChunk(fileData: File, uploadInfo: UploadInfo, destinationDirectory: FileSystemItem): TPromise<any>;

    /**
     */
    abortFileUpload(fileData: File, uploadInfo: UploadInfo, destinationDirectory: FileSystemItem): TPromise<any>;

    /**
     */
    downloadItems(items: Array<FileSystemItem>): void;

    /**
     */
    getItemsContent(items: Array<FileSystemItem>): TPromise<any>;
}
