/**
 * DevExtreme (renovation/ui/scroll_view/scrollbar.js)
 * Version: 21.1.0 (build 21089-0329)
 * Build date: Tue Mar 30 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.Scrollbar = exports.ScrollbarPropsType = exports.viewFunction = void 0;
var _inferno = require("inferno");
var _effect = require("devextreme-generator/modules/inferno/effect");
var _base_component = require("devextreme-generator/modules/inferno/base_component");
var _widget = require("../common/widget");
var _combine_classes = require("../../utils/combine_classes");
var _dom_adapter = _interopRequireDefault(require("../../../core/dom_adapter"));
var _type = require("../../../core/utils/type");
var _index = require("../../../events/utils/index");
var _scrollbar_props = require("./scrollbar_props");
var _scrollable_utils = require("./scrollable_utils");
var _short = require("../../../events/short");
var _scrollable_simulated_props = require("./scrollable_simulated_props");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _objectWithoutProperties(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = _objectWithoutPropertiesLoose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for (i = 0; i < sourceSymbolKeys.length; i++) {
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) {
                continue
            }
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) {
                continue
            }
            target[key] = source[key]
        }
    }
    return target
}

function _objectWithoutPropertiesLoose(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) {
            continue
        }
        target[key] = source[key]
    }
    return target
}

function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        })
    } else {
        obj[key] = value
    }
    return obj
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}

function _assertThisInitialized(self) {
    if (void 0 === self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return self
}

function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf(subClass, superClass)
}

function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function(o, p) {
        o.__proto__ = p;
        return o
    };
    return _setPrototypeOf(o, p)
}

function _extends() {
    _extends = Object.assign || function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}
var OUT_BOUNDS_ACCELERATION = .5;
var SCROLLABLE_SCROLLBAR_ACTIVE_CLASS = "dx-scrollable-scrollbar-active";
var SCROLLABLE_SCROLL_CLASS = "dx-scrollable-scroll";
var SCROLLABLE_SCROLL_CONTENT_CLASS = "dx-scrollable-scroll-content";
var HOVER_ENABLED_STATE = "dx-scrollbar-hoverable";
var MAX_OFFSET = 0;
var THUMB_MIN_SIZE = 15;
var HIDE_SCROLLBAR_TIMEOUT = 500;
var viewFunction = function(viewModel) {
    var cssClasses = viewModel.cssClasses,
        hoverStateEnabled = viewModel.hoverStateEnabled,
        isVisible = viewModel.isVisible,
        onHoverEndHandler = viewModel.onHoverEndHandler,
        onHoverStartHandler = viewModel.onHoverStartHandler,
        activeStateEnabled = viewModel.props.activeStateEnabled,
        restAttributes = viewModel.restAttributes,
        scrollRef = viewModel.scrollRef,
        scrollbarRef = viewModel.scrollbarRef,
        styles = viewModel.styles;
    return (0, _inferno.normalizeProps)((0, _inferno.createComponentVNode)(2, _widget.Widget, _extends({
        rootElementRef: scrollbarRef,
        classes: cssClasses,
        activeStateEnabled: activeStateEnabled,
        hoverStateEnabled: hoverStateEnabled,
        visible: isVisible,
        onHoverStart: onHoverStartHandler,
        onHoverEnd: onHoverEndHandler
    }, restAttributes, {
        children: (0, _inferno.createVNode)(1, "div", viewModel.scrollClasses, (0, _inferno.createVNode)(1, "div", SCROLLABLE_SCROLL_CONTENT_CLASS), 2, {
            style: styles
        }, null, scrollRef)
    })))
};
exports.viewFunction = viewFunction;
var ScrollbarPropsType = {
    activeStateEnabled: _scrollbar_props.ScrollbarProps.activeStateEnabled,
    expandable: _scrollbar_props.ScrollbarProps.expandable,
    direction: _scrollbar_props.ScrollbarProps.direction,
    scaleRatio: _scrollbar_props.ScrollbarProps.scaleRatio,
    containerSize: _scrollbar_props.ScrollbarProps.containerSize,
    contentSize: _scrollbar_props.ScrollbarProps.contentSize,
    baseContainerSize: _scrollbar_props.ScrollbarProps.baseContainerSize,
    baseContentSize: _scrollbar_props.ScrollbarProps.baseContentSize,
    scrollableOffset: _scrollbar_props.ScrollbarProps.scrollableOffset,
    scrollByThumb: _scrollbar_props.ScrollbarProps.scrollByThumb,
    scrollScrollbar: _scrollbar_props.ScrollbarProps.scrollScrollbar,
    isScrollableHovered: _scrollbar_props.ScrollbarProps.isScrollableHovered,
    bounceEnabled: _scrollbar_props.ScrollbarProps.bounceEnabled,
    forceVisibility: _scrollbar_props.ScrollbarProps.forceVisibility,
    showScrollbar: _scrollbar_props.ScrollbarProps.showScrollbar,
    scrollByContent: _scrollbar_props.ScrollbarProps.scrollByContent,
    useKeyboard: _scrollbar_props.ScrollbarProps.useKeyboard,
    forceGeneratePockets: _scrollbar_props.ScrollbarProps.forceGeneratePockets,
    needScrollViewContentWrapper: _scrollbar_props.ScrollbarProps.needScrollViewContentWrapper,
    defaultContentTranslateOffset: _scrollable_simulated_props.ScrollableSimulatedProps.contentTranslateOffset
};
exports.ScrollbarPropsType = ScrollbarPropsType;
var Scrollbar = function(_InfernoComponent) {
    _inheritsLoose(Scrollbar, _InfernoComponent);

    function Scrollbar(props) {
        var _this;
        _this = _InfernoComponent.call(this, props) || this;
        _this._currentState = null;
        _this.location = 0;
        _this.thumbScrolling = false;
        _this.crossThumbScrolling = false;
        _this.prevThumbRatio = 1;
        _this.scrollbarRef = (0, _inferno.createRef)();
        _this.scrollRef = (0, _inferno.createRef)();
        _this.state = {
            showOnScrollByWheel: void 0,
            hovered: false,
            active: false,
            visibility: false,
            scrollLocation: 0,
            contentTranslateOffset: void 0 !== _this.props.contentTranslateOffset ? _this.props.contentTranslateOffset : _this.props.defaultContentTranslateOffset
        };
        _this.pointerDownEffect = _this.pointerDownEffect.bind(_assertThisInitialized(_this));
        _this.pointerUpEffect = _this.pointerUpEffect.bind(_assertThisInitialized(_this));
        _this.isThumb = _this.isThumb.bind(_assertThisInitialized(_this));
        _this.isScrollbar = _this.isScrollbar.bind(_assertThisInitialized(_this));
        _this.validateEvent = _this.validateEvent.bind(_assertThisInitialized(_this));
        _this.reachedMin = _this.reachedMin.bind(_assertThisInitialized(_this));
        _this.reachedMax = _this.reachedMax.bind(_assertThisInitialized(_this));
        _this.getLocation = _this.getLocation.bind(_assertThisInitialized(_this));
        _this.getScrollLocation = _this.getScrollLocation.bind(_assertThisInitialized(_this));
        _this.setLocation = _this.setLocation.bind(_assertThisInitialized(_this));
        _this.inBounds = _this.inBounds.bind(_assertThisInitialized(_this));
        _this.boundLocation = _this.boundLocation.bind(_assertThisInitialized(_this));
        _this.getMinOffset = _this.getMinOffset.bind(_assertThisInitialized(_this));
        _this.getMaxOffset = _this.getMaxOffset.bind(_assertThisInitialized(_this));
        _this.initHandler = _this.initHandler.bind(_assertThisInitialized(_this));
        _this.startHandler = _this.startHandler.bind(_assertThisInitialized(_this));
        _this.moveHandler = _this.moveHandler.bind(_assertThisInitialized(_this));
        _this.hide = _this.hide.bind(_assertThisInitialized(_this));
        _this.endHandler = _this.endHandler.bind(_assertThisInitialized(_this));
        _this.stopHandler = _this.stopHandler.bind(_assertThisInitialized(_this));
        _this.scrollByHandler = _this.scrollByHandler.bind(_assertThisInitialized(_this));
        _this.scrollComplete = _this.scrollComplete.bind(_assertThisInitialized(_this));
        _this.scrollToBounds = _this.scrollToBounds.bind(_assertThisInitialized(_this));
        _this.stopComplete = _this.stopComplete.bind(_assertThisInitialized(_this));
        _this.resetThumbScrolling = _this.resetThumbScrolling.bind(_assertThisInitialized(_this));
        _this.scrollBy = _this.scrollBy.bind(_assertThisInitialized(_this));
        _this.stopScrolling = _this.stopScrolling.bind(_assertThisInitialized(_this));
        _this.prepareThumbScrolling = _this.prepareThumbScrolling.bind(_assertThisInitialized(_this));
        _this.moveToMouseLocation = _this.moveToMouseLocation.bind(_assertThisInitialized(_this));
        _this.scrollStep = _this.scrollStep.bind(_assertThisInitialized(_this));
        _this.suppressBounce = _this.suppressBounce.bind(_assertThisInitialized(_this));
        _this.move = _this.move.bind(_assertThisInitialized(_this));
        _this.moveScrollbar = _this.moveScrollbar.bind(_assertThisInitialized(_this));
        _this.moveContent = _this.moveContent.bind(_assertThisInitialized(_this));
        _this.containerToContentRatio = _this.containerToContentRatio.bind(_assertThisInitialized(_this));
        _this.baseContainerToContentRatio = _this.baseContainerToContentRatio.bind(_assertThisInitialized(_this));
        _this.feedbackOn = _this.feedbackOn.bind(_assertThisInitialized(_this));
        _this.feedbackOff = _this.feedbackOff.bind(_assertThisInitialized(_this));
        _this.onHoverStartHandler = _this.onHoverStartHandler.bind(_assertThisInitialized(_this));
        _this.onHoverEndHandler = _this.onHoverEndHandler.bind(_assertThisInitialized(_this));
        return _this
    }
    var _proto = Scrollbar.prototype;
    _proto.createEffects = function() {
        return [new _effect.InfernoEffect(this.pointerDownEffect, []), new _effect.InfernoEffect(this.pointerUpEffect, [])]
    };
    _proto.updateEffects = function() {
        var _this$_effects$, _this$_effects$2;
        null === (_this$_effects$ = this._effects[0]) || void 0 === _this$_effects$ ? void 0 : _this$_effects$.update([]);
        null === (_this$_effects$2 = this._effects[1]) || void 0 === _this$_effects$2 ? void 0 : _this$_effects$2.update([])
    };
    _proto.set_showOnScrollByWheel = function(value) {
        var _this2 = this;
        this.setState((function(state) {
            _this2._currentState = state;
            var newValue = value();
            _this2._currentState = null;
            return {
                showOnScrollByWheel: newValue
            }
        }))
    };
    _proto.set_hovered = function(value) {
        var _this3 = this;
        this.setState((function(state) {
            _this3._currentState = state;
            var newValue = value();
            _this3._currentState = null;
            return {
                hovered: newValue
            }
        }))
    };
    _proto.set_active = function(value) {
        var _this4 = this;
        this.setState((function(state) {
            _this4._currentState = state;
            var newValue = value();
            _this4._currentState = null;
            return {
                active: newValue
            }
        }))
    };
    _proto.set_visibility = function(value) {
        var _this5 = this;
        this.setState((function(state) {
            _this5._currentState = state;
            var newValue = value();
            _this5._currentState = null;
            return {
                visibility: newValue
            }
        }))
    };
    _proto.set_scrollLocation = function(value) {
        var _this6 = this;
        this.setState((function(state) {
            _this6._currentState = state;
            var newValue = value();
            _this6._currentState = null;
            return {
                scrollLocation: newValue
            }
        }))
    };
    _proto.set_contentTranslateOffset = function(value) {
        var _this7 = this;
        this.setState((function(state) {
            var _this7$props$contentT, _this7$props;
            _this7._currentState = state;
            var newValue = value();
            null === (_this7$props$contentT = (_this7$props = _this7.props).contentTranslateOffsetChange) || void 0 === _this7$props$contentT ? void 0 : _this7$props$contentT.call(_this7$props, newValue);
            _this7._currentState = null;
            return {
                contentTranslateOffset: newValue
            }
        }))
    };
    _proto.pointerDownEffect = function() {
        var _this8 = this;
        _short.dxPointerDown.on(this.scrollRef.current, (function() {
            _this8.feedbackOn()
        }), {
            namespace: "dxScrollbar"
        });
        return function() {
            return _short.dxPointerDown.off(_this8.scrollRef.current, {
                namespace: "dxScrollbar"
            })
        }
    };
    _proto.pointerUpEffect = function() {
        var _this9 = this;
        _short.dxPointerUp.on(_dom_adapter.default.getDocument(), (function() {
            _this9.feedbackOff()
        }), {
            namespace: "dxScrollbar"
        });
        return function() {
            return _short.dxPointerUp.off(_this9.scrollRef.current, {
                namespace: "dxScrollbar"
            })
        }
    };
    _proto.hide = function() {
        var _this10 = this;
        this.set_visibility((function() {
            return false
        }));
        if ((0, _type.isDefined)(this.showOnScrollByWheel) && "onScroll" === this.props.showScrollbar) {
            setTimeout((function() {
                _this10.set_showOnScrollByWheel((function() {
                    return
                }))
            }), HIDE_SCROLLBAR_TIMEOUT)
        }
    };
    _proto.scrollToBounds = function() {
        var _this$props$onAnimato, _this$props;
        if (this.inBounds()) {
            return
        }
        null === (_this$props$onAnimato = (_this$props = this.props).onAnimatorStart) || void 0 === _this$props$onAnimato ? void 0 : _this$props$onAnimato.call(_this$props, "bounce")
    };
    _proto.resetThumbScrolling = function() {
        this.thumbScrolling = false;
        this.crossThumbScrolling = false
    };
    _proto.scrollBy = function(delta) {
        var distance = delta[this.axis];
        if (!this.inBounds()) {
            distance *= OUT_BOUNDS_ACCELERATION
        }
        this.scrollStep(distance)
    };
    _proto.stopScrolling = function() {
        var _this$props$onAnimato2, _this$props2;
        this.hide();
        null === (_this$props$onAnimato2 = (_this$props2 = this.props).onAnimatorCancel) || void 0 === _this$props$onAnimato2 ? void 0 : _this$props$onAnimato2.call(_this$props2)
    };
    _proto.prepareThumbScrolling = function(e, currentCrossThumbScrolling) {
        if ((0, _index.isDxMouseWheelEvent)(e.originalEvent)) {
            if ("onScroll" === this.props.showScrollbar) {
                this.set_showOnScrollByWheel((function() {
                    return true
                }))
            }
            return
        }
        var target = e.originalEvent.target;
        var scrollByThumb = this.props.scrollByThumb;
        var scrollbarClicked = scrollByThumb && this.isScrollbar(target);
        if (scrollbarClicked) {
            this.moveToMouseLocation(e)
        }
        var currentThumbScrolling = scrollbarClicked || scrollByThumb && this.isThumb(target);
        this.thumbScrolling = currentThumbScrolling;
        this.crossThumbScrolling = !currentThumbScrolling && currentCrossThumbScrolling;
        if (currentThumbScrolling) {
            this.feedbackOn()
        }
    };
    _proto.moveToMouseLocation = function(e) {
        var mouseLocation = e["page".concat(this.axis.toUpperCase())] - this.props.scrollableOffset;
        var location = this.getLocation() + mouseLocation / this.containerToContentRatio() - this.props.containerSize / 2;
        this.scrollStep(-Math.round(location))
    };
    _proto.suppressBounce = function() {
        if (this.props.bounceEnabled || this.inBounds()) {
            return
        }
        this.setLocation(this.boundLocation())
    };
    _proto.moveContent = function() {
        var _this$props$contentPo, _this$props3, _this$props$contentTr, _this$props4;
        var location = this.getLocation();
        var currentTranslateOffset;
        if (location > 0) {
            currentTranslateOffset = location
        } else if (location <= this.minOffset) {
            currentTranslateOffset = location - this.minOffset
        } else {
            currentTranslateOffset = location % 1
        }
        null === (_this$props$contentPo = (_this$props3 = this.props).contentPositionChange) || void 0 === _this$props$contentPo ? void 0 : _this$props$contentPo.call(_this$props3, this.fullScrollProp, location, this.props.scaleRatio);
        null === (_this$props$contentTr = (_this$props4 = this.props).contentTranslateOffsetChange) || void 0 === _this$props$contentTr ? void 0 : _this$props$contentTr.call(_this$props4, _defineProperty({}, this.scrollProp, currentTranslateOffset))
    };
    _proto.containerToContentRatio = function() {
        var _this$props5 = this.props,
            containerSize = _this$props5.containerSize,
            contentSize = _this$props5.contentSize;
        return contentSize ? containerSize / contentSize : containerSize
    };
    _proto.baseContainerToContentRatio = function() {
        var _this$props6 = this.props,
            baseContainerSize = _this$props6.baseContainerSize,
            baseContentSize = _this$props6.baseContentSize;
        return baseContentSize ? baseContainerSize / baseContentSize : baseContainerSize
    };
    _proto.feedbackOn = function() {
        this.set_active((function() {
            return true
        }))
    };
    _proto.feedbackOff = function() {
        this.set_active((function() {
            return false
        }))
    };
    _proto.onHoverStartHandler = function() {
        if ("onHover" === this.props.showScrollbar) {
            this.set_hovered((function() {
                return true
            }))
        }
    };
    _proto.onHoverEndHandler = function() {
        if ("onHover" === this.props.showScrollbar) {
            this.set_hovered((function() {
                return false
            }))
        }
    };
    _proto.isThumb = function(element) {
        var _this$scrollbarRef$cu, _this$scrollbarRef$cu2;
        return (null === (_this$scrollbarRef$cu = this.scrollbarRef.current) || void 0 === _this$scrollbarRef$cu ? void 0 : _this$scrollbarRef$cu.querySelector(".".concat(SCROLLABLE_SCROLL_CLASS))) === element || (null === (_this$scrollbarRef$cu2 = this.scrollbarRef.current) || void 0 === _this$scrollbarRef$cu2 ? void 0 : _this$scrollbarRef$cu2.querySelector(".".concat(SCROLLABLE_SCROLL_CONTENT_CLASS))) === element
    };
    _proto.isScrollbar = function(element) {
        return element === this.scrollbarRef.current
    };
    _proto.validateEvent = function(e) {
        var target = e.originalEvent.target;
        return this.isThumb(target) || this.isScrollbar(target)
    };
    _proto.reachedMin = function() {
        return this.getLocation() <= this.minOffset
    };
    _proto.reachedMax = function() {
        return this.getLocation() >= this.maxOffset
    };
    _proto.getLocation = function() {
        return this.location
    };
    _proto.getScrollLocation = function() {
        return this.scrollLocation
    };
    _proto.setLocation = function(value) {
        this.location = value
    };
    _proto.inBounds = function() {
        return this.boundLocation() === this.getLocation()
    };
    _proto.boundLocation = function(value) {
        var currentLocation = (0, _type.isDefined)(value) ? value : this.getLocation();
        return Math.max(Math.min(currentLocation, this.maxOffset), this.minOffset)
    };
    _proto.getMinOffset = function() {
        return this.minOffset
    };
    _proto.getMaxOffset = function() {
        return this.maxOffset
    };
    _proto.initHandler = function(e, crossThumbScrolling) {
        this.stopScrolling();
        this.prepareThumbScrolling(e, crossThumbScrolling)
    };
    _proto.startHandler = function() {
        this.set_visibility((function() {
            return true
        }))
    };
    _proto.moveHandler = function(delta) {
        if (this.crossThumbScrolling) {
            return
        }
        var distance = delta;
        if (this.thumbScrolling) {
            distance[this.axis] = -Math.round(distance[this.axis] / this.containerToContentRatio())
        }
        this.scrollBy(distance)
    };
    _proto.endHandler = function(velocity) {
        var _this$props$onAnimato3, _this$props7;
        null === (_this$props$onAnimato3 = (_this$props7 = this.props).onAnimatorStart) || void 0 === _this$props$onAnimato3 ? void 0 : _this$props$onAnimato3.call(_this$props7, "inertia", velocity[this.axis], this.thumbScrolling, this.crossThumbScrolling);
        this.resetThumbScrolling()
    };
    _proto.stopHandler = function() {
        this.hide();
        if (this.thumbScrolling) {
            this.scrollComplete()
        } else {
            this.scrollToBounds()
        }
        this.resetThumbScrolling()
    };
    _proto.scrollByHandler = function(delta) {
        this.scrollBy(delta);
        this.scrollComplete()
    };
    _proto.scrollComplete = function() {
        if (this.inBounds()) {
            this.hide()
        }
        this.scrollToBounds()
    };
    _proto.stopComplete = function() {};
    _proto.scrollStep = function(delta) {
        this.setLocation(this.getLocation() + delta);
        this.suppressBounce();
        this.move()
    };
    _proto.move = function(location) {
        this.moveScrollbar(location);
        this.moveContent()
    };
    _proto.moveScrollbar = function(location) {
        var currentLocation = void 0 !== location ? location * this.props.scaleRatio : this.getLocation();
        this.setLocation(currentLocation);
        this.set_scrollLocation((function() {
            return currentLocation
        }))
    };
    _proto.render = function() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props, {
                contentTranslateOffset: this.__state_contentTranslateOffset
            }),
            showOnScrollByWheel: this.showOnScrollByWheel,
            hovered: this.hovered,
            active: this.active,
            visibility: this.visibility,
            scrollLocation: this.scrollLocation,
            scrollbarRef: this.scrollbarRef,
            scrollRef: this.scrollRef,
            scrollTranslateOffset: this.scrollTranslateOffset,
            scrollPosition: this.scrollPosition,
            minOffset: this.minOffset,
            maxOffset: this.maxOffset,
            axis: this.axis,
            scrollProp: this.scrollProp,
            fullScrollProp: this.fullScrollProp,
            hide: this.hide,
            scrollToBounds: this.scrollToBounds,
            resetThumbScrolling: this.resetThumbScrolling,
            scrollBy: this.scrollBy,
            stopScrolling: this.stopScrolling,
            prepareThumbScrolling: this.prepareThumbScrolling,
            moveToMouseLocation: this.moveToMouseLocation,
            suppressBounce: this.suppressBounce,
            moveContent: this.moveContent,
            thumbSize: this.thumbSize,
            thumbRatio: this.thumbRatio,
            containerToContentRatio: this.containerToContentRatio,
            baseContainerToContentRatio: this.baseContainerToContentRatio,
            dimension: this.dimension,
            feedbackOn: this.feedbackOn,
            feedbackOff: this.feedbackOff,
            onHoverStartHandler: this.onHoverStartHandler,
            onHoverEndHandler: this.onHoverEndHandler,
            cssClasses: this.cssClasses,
            styles: this.styles,
            scrollClasses: this.scrollClasses,
            isVisible: this.isVisible,
            visible: this.visible,
            hoverStateEnabled: this.hoverStateEnabled,
            restAttributes: this.restAttributes
        })
    };
    _createClass(Scrollbar, [{
        key: "showOnScrollByWheel",
        get: function() {
            var state = this._currentState || this.state;
            return state.showOnScrollByWheel
        }
    }, {
        key: "hovered",
        get: function() {
            var state = this._currentState || this.state;
            return state.hovered
        }
    }, {
        key: "active",
        get: function() {
            var state = this._currentState || this.state;
            return state.active
        }
    }, {
        key: "visibility",
        get: function() {
            var state = this._currentState || this.state;
            return state.visibility
        }
    }, {
        key: "scrollLocation",
        get: function() {
            var state = this._currentState || this.state;
            return state.scrollLocation
        }
    }, {
        key: "__state_contentTranslateOffset",
        get: function() {
            var state = this._currentState || this.state;
            return void 0 !== this.props.contentTranslateOffset ? this.props.contentTranslateOffset : state.contentTranslateOffset
        }
    }, {
        key: "scrollTranslateOffset",
        get: function() {
            return _extends({}, {
                left: 0,
                top: 0
            }, _defineProperty({}, this.scrollProp, this.scrollPosition))
        }
    }, {
        key: "scrollPosition",
        get: function() {
            var _this11 = this;
            var thumbRatioChanged = Math.abs(this.thumbRatio - this.prevThumbRatio) > .01;
            if (thumbRatioChanged) {
                this.set_scrollLocation((function() {
                    return _this11.boundLocation(_this11.scrollLocation)
                }));
                this.prevThumbRatio = this.thumbRatio
            }
            return -this.scrollLocation * this.thumbRatio
        }
    }, {
        key: "minOffset",
        get: function() {
            return Math.round(-Math.max(this.props.contentSize - this.props.containerSize, 0))
        }
    }, {
        key: "maxOffset",
        get: function() {
            return MAX_OFFSET
        }
    }, {
        key: "axis",
        get: function() {
            return this.props.direction === _scrollable_utils.DIRECTION_HORIZONTAL ? "x" : "y"
        }
    }, {
        key: "scrollProp",
        get: function() {
            return this.props.direction === _scrollable_utils.DIRECTION_HORIZONTAL ? "left" : "top"
        }
    }, {
        key: "fullScrollProp",
        get: function() {
            return this.props.direction === _scrollable_utils.DIRECTION_HORIZONTAL ? "scrollLeft" : "scrollTop"
        }
    }, {
        key: "thumbSize",
        get: function() {
            var _this$props8 = this.props,
                containerSize = _this$props8.containerSize,
                scaleRatio = _this$props8.scaleRatio;
            var size = Math.round(Math.max(Math.round(containerSize * this.containerToContentRatio()), THUMB_MIN_SIZE));
            return size / scaleRatio
        }
    }, {
        key: "thumbRatio",
        get: function() {
            var _this$props9 = this.props,
                containerSize = _this$props9.containerSize,
                contentSize = _this$props9.contentSize,
                scaleRatio = _this$props9.scaleRatio;
            if (contentSize) {
                return (containerSize - this.thumbSize) / (scaleRatio * (contentSize - containerSize))
            }
            return 1
        }
    }, {
        key: "dimension",
        get: function() {
            return this.props.direction === _scrollable_utils.DIRECTION_HORIZONTAL ? "width" : "height"
        }
    }, {
        key: "cssClasses",
        get: function() {
            var _classesMap;
            var direction = this.props.direction;
            var classesMap = (_classesMap = {}, _defineProperty(_classesMap, _scrollable_utils.SCROLLABLE_SCROLLBAR_CLASS, true), _defineProperty(_classesMap, "dx-scrollbar-".concat(direction), true), _defineProperty(_classesMap, SCROLLABLE_SCROLLBAR_ACTIVE_CLASS, !!this.active), _defineProperty(_classesMap, HOVER_ENABLED_STATE, !!this.hoverStateEnabled), _classesMap);
            return (0, _combine_classes.combineClasses)(classesMap)
        }
    }, {
        key: "styles",
        get: function() {
            var _ref;
            return _ref = {}, _defineProperty(_ref, this.dimension, this.thumbSize || THUMB_MIN_SIZE), _defineProperty(_ref, "transform", "never" === this.props.showScrollbar ? "none" : "translate(".concat(this.scrollTranslateOffset.left, "px, ").concat(this.scrollTranslateOffset.top, "px)")), _ref
        }
    }, {
        key: "scrollClasses",
        get: function() {
            var _combineClasses;
            return (0, _combine_classes.combineClasses)((_combineClasses = {}, _defineProperty(_combineClasses, SCROLLABLE_SCROLL_CLASS, true), _defineProperty(_combineClasses, "dx-state-invisible", !this.visible), _combineClasses))
        }
    }, {
        key: "isVisible",
        get: function() {
            return "never" !== this.props.showScrollbar && this.baseContainerToContentRatio() < 1
        }
    }, {
        key: "visible",
        get: function() {
            var _this$props10 = this.props,
                forceVisibility = _this$props10.forceVisibility,
                showScrollbar = _this$props10.showScrollbar;
            if (!this.isVisible) {
                return false
            }
            if ("onHover" === showScrollbar) {
                return this.visibility || this.props.isScrollableHovered || this.hovered
            }
            if ("always" === showScrollbar) {
                return true
            }
            return forceVisibility || this.visibility || !!this.showOnScrollByWheel
        }
    }, {
        key: "hoverStateEnabled",
        get: function() {
            var _this$props11 = this.props,
                scrollByThumb = _this$props11.scrollByThumb,
                showScrollbar = _this$props11.showScrollbar;
            return ("onHover" === showScrollbar || "always" === showScrollbar) && scrollByThumb
        }
    }, {
        key: "restAttributes",
        get: function() {
            var _this$props$contentTr3 = _extends({}, this.props, {
                    contentTranslateOffset: this.__state_contentTranslateOffset
                }),
                restProps = (_this$props$contentTr3.activeStateEnabled, _this$props$contentTr3.baseContainerSize, _this$props$contentTr3.baseContentSize, _this$props$contentTr3.bounceEnabled, _this$props$contentTr3.children, _this$props$contentTr3.classes, _this$props$contentTr3.containerSize, _this$props$contentTr3.contentPositionChange, _this$props$contentTr3.contentSize, _this$props$contentTr3.contentTranslateOffset, _this$props$contentTr3.contentTranslateOffsetChange, _this$props$contentTr3.defaultContentTranslateOffset, _this$props$contentTr3.direction, _this$props$contentTr3.expandable, _this$props$contentTr3.forceGeneratePockets, _this$props$contentTr3.forceVisibility, _this$props$contentTr3.hoverStateEnabled, _this$props$contentTr3.isScrollableHovered, _this$props$contentTr3.needScrollViewContentWrapper, _this$props$contentTr3.onAnimatorCancel, _this$props$contentTr3.onAnimatorStart, _this$props$contentTr3.onScroll, _this$props$contentTr3.onUpdated, _this$props$contentTr3.scaleRatio, _this$props$contentTr3.scrollByContent, _this$props$contentTr3.scrollByThumb, _this$props$contentTr3.scrollScrollbar, _this$props$contentTr3.scrollableOffset, _this$props$contentTr3.showScrollbar, _this$props$contentTr3.useKeyboard, _objectWithoutProperties(_this$props$contentTr3, ["activeStateEnabled", "baseContainerSize", "baseContentSize", "bounceEnabled", "children", "classes", "containerSize", "contentPositionChange", "contentSize", "contentTranslateOffset", "contentTranslateOffsetChange", "defaultContentTranslateOffset", "direction", "expandable", "forceGeneratePockets", "forceVisibility", "hoverStateEnabled", "isScrollableHovered", "needScrollViewContentWrapper", "onAnimatorCancel", "onAnimatorStart", "onScroll", "onUpdated", "scaleRatio", "scrollByContent", "scrollByThumb", "scrollScrollbar", "scrollableOffset", "showScrollbar", "useKeyboard"]));
            return restProps
        }
    }]);
    return Scrollbar
}(_base_component.InfernoComponent);
exports.Scrollbar = Scrollbar;
Scrollbar.defaultProps = _extends({}, ScrollbarPropsType);
