/**
* DevExtreme (ui/data_grid.d.ts)
* Version: 21.1.0 (build 21089-0329)
* Build date: Tue Mar 30 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
  TElement,
  TElementsArray
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import {
    template
} from '../core/templates/template';

import Store from '../data/abstract_store';

import DataSource, {
    DataSourceOptions
} from '../data/data_source';

import {
    TEvent
} from '../events/index';

import {
    ExcelDataGridCell
} from '../excel_exporter';

import {
    ExcelFont
} from '../exporter/excel/excel.doc_comments';

import dxDraggable from './draggable';

import {
    dxFilterBuilderOptions
} from './filter_builder';

import {
    dxFormOptions,
    dxFormSimpleItem
} from './form';

import {
    dxPopupOptions
} from './popup';

import dxScrollable from './scroll_view/ui.scrollable';

import dxSortable from './sortable';

import {
    dxToolbarOptions
} from './toolbar';

import {
    AsyncRule,
    CompareRule,
    CustomRule,
    EmailRule,
    NumericRule,
    PatternRule,
    RangeRule,
    RequiredRule,
    StringLengthRule
} from './validation_rules';

import Widget, {
    format,
    WidgetOptions
} from './widget/ui.widget';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface GridBaseOptions<T = GridBase> extends WidgetOptions<T> {
    /**
     */
    allowColumnReordering?: boolean;
    /**
     */
    allowColumnResizing?: boolean;
    /**
     */
    autoNavigateToFocusedRow?: boolean;
    /**
     */
    cacheEnabled?: boolean;
    /**
     */
    cellHintEnabled?: boolean;
    /**
     */
    columnAutoWidth?: boolean;
    /**
     */
    columnChooser?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      allowSearch?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      emptyPanelText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      enabled?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      height?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      mode?: 'dragAndDrop' | 'select',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      searchTimeout?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      title?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     */
    columnFixing?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      enabled?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      texts?: {
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        fix?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        leftPosition?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        rightPosition?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        unfix?: string
      }
    };
    /**
     */
    columnHidingEnabled?: boolean;
    /**
     */
    columnMinWidth?: number;
    /**
     */
    columnResizingMode?: 'nextColumn' | 'widget';
    /**
     */
    columnWidth?: number;
    /**
     */
    columns?: Array<GridBaseColumn | string>;
    /**
     */
    dataSource?: string | Array<any> | DataSource | DataSourceOptions;
    /**
     */
    dateSerializationFormat?: string;
    /**
     */
    editing?: GridBaseEditing;
    /**
     */
    errorRowEnabled?: boolean;
    /**
     */
    filterBuilder?: dxFilterBuilderOptions;
    /**
     */
    filterBuilderPopup?: dxPopupOptions;
    /**
     */
    filterPanel?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      customizeText?: ((e: { component?: T, filterValue?: any, text?: string }) => string),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      filterEnabled?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      texts?: {
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        clearFilter?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        createFilter?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        filterEnabledHint?: string
      },
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean
    };
    /**
     */
    filterRow?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      applyFilter?: 'auto' | 'onClick',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      applyFilterText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      betweenEndText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      betweenStartText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      operationDescriptions?: {
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        between?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        contains?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        endsWith?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        equal?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        greaterThan?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        greaterThanOrEqual?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        lessThan?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        lessThanOrEqual?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        notContains?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        notEqual?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        startsWith?: string
      },
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      resetOperationText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      showAllText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      showOperationChooser?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean
    };
    /**
     */
    filterSyncEnabled?: boolean | 'auto';
    /**
     */
    filterValue?: string | Array<any> | Function;
    /**
     */
    focusedColumnIndex?: number;
    /**
     */
    focusedRowEnabled?: boolean;
    /**
     */
    focusedRowIndex?: number;
    /**
     */
    focusedRowKey?: any;
    /**
     */
    headerFilter?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      allowSearch?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      height?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      searchTimeout?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      texts?: {
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        cancel?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        emptyValue?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        ok?: string
      },
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     */
    highlightChanges?: boolean;
    /**
     */
    keyboardNavigation?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      editOnKeyPress?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      enabled?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      enterKeyAction?: 'startEdit' | 'moveFocus',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      enterKeyDirection?: 'none' | 'column' | 'row'
    };
    /**
     */
    loadPanel?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      enabled?: boolean | 'auto',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      height?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      indicatorSrc?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      shading?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      shadingColor?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      showIndicator?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      showPane?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      text?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     */
    noDataText?: string;
    /**
     */
    onAdaptiveDetailRowPreparing?: ((e: { component?: T, element?: TElement, model?: any, formOptions?: any }) => void);
    /**
     */
    onDataErrorOccurred?: ((e: { component?: T, element?: TElement, model?: any, error?: Error }) => void);
    /**
     */
    onEditCanceled?: ((e: { component?: T, element?: TElement, model?: any, changes?: Array<any> }) => void);
    /**
     */
    onEditCanceling?: ((e: { component?: T, element?: TElement, model?: any, changes?: Array<any>, cancel?: boolean }) => void);
    /**
     */
    onInitNewRow?: ((e: { component?: T, element?: TElement, model?: any, data?: any, promise?: TPromise<void> }) => void);
    /**
     */
    onKeyDown?: ((e: { component?: T, element?: TElement, model?: any, event?: TEvent, handled?: boolean }) => void);
    /**
     */
    onRowCollapsed?: ((e: { component?: T, element?: TElement, model?: any, key?: any }) => void);
    /**
     */
    onRowCollapsing?: ((e: { component?: T, element?: TElement, model?: any, key?: any, cancel?: boolean }) => void);
    /**
     */
    onRowExpanded?: ((e: { component?: T, element?: TElement, model?: any, key?: any }) => void);
    /**
     */
    onRowExpanding?: ((e: { component?: T, element?: TElement, model?: any, key?: any, cancel?: boolean }) => void);
    /**
     */
    onRowInserted?: ((e: { component?: T, element?: TElement, model?: any, data?: any, key?: any, error?: Error }) => void);
    /**
     */
    onRowInserting?: ((e: { component?: T, element?: TElement, model?: any, data?: any, cancel?: boolean | TPromise<void> }) => void);
    /**
     */
    onRowRemoved?: ((e: { component?: T, element?: TElement, model?: any, data?: any, key?: any, error?: Error }) => void);
    /**
     */
    onRowRemoving?: ((e: { component?: T, element?: TElement, model?: any, data?: any, key?: any, cancel?: boolean | TPromise<void> }) => void);
    /**
     */
    onRowUpdated?: ((e: { component?: T, element?: TElement, model?: any, data?: any, key?: any, error?: Error }) => void);
    /**
     */
    onRowUpdating?: ((e: { component?: T, element?: TElement, model?: any, oldData?: any, newData?: any, key?: any, cancel?: boolean | TPromise<void> }) => void);
    /**
     */
    onRowValidating?: ((e: { component?: T, element?: TElement, model?: any, brokenRules?: Array<RequiredRule | NumericRule | RangeRule | StringLengthRule | CustomRule | CompareRule | PatternRule | EmailRule | AsyncRule>, isValid?: boolean, key?: any, newData?: any, oldData?: any, errorText?: string, promise?: TPromise<void> }) => void);
    /**
     */
    onSaved?: ((e: { component?: T, element?: TElement, model?: any, changes?: Array<any> }) => void);
    /**
     */
    onSaving?: ((e: { component?: T, element?: TElement, model?: any, changes?: Array<any>, promise?: TPromise<void>, cancel?: boolean }) => void);
    /**
     */
    onSelectionChanged?: ((e: { component?: T, element?: TElement, model?: any, currentSelectedRowKeys?: Array<any>, currentDeselectedRowKeys?: Array<any>, selectedRowKeys?: Array<any>, selectedRowsData?: Array<any> }) => void);
    /**
     */
    onToolbarPreparing?: ((e: { component?: T, element?: TElement, model?: any, toolbarOptions?: dxToolbarOptions }) => void);
    /**
     */
    pager?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      allowedPageSizes?: Array<(number | 'all')> | 'auto',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      displayMode?: 'adaptive' | 'compact' | 'full',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      infoText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      showInfo?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      showNavigationButtons?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      showPageSizeSelector?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean | 'auto'
    };
    /**
     */
    paging?: GridBasePaging;
    /**
     */
    renderAsync?: boolean;
    /**
     */
    repaintChangesOnly?: boolean;
    /**
     */
    rowAlternationEnabled?: boolean;
    /**
     */
    rowDragging?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      allowDropInsideItem?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      allowReordering?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      autoScroll?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      boundary?: string | TElement,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      container?: string | TElement,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      cursorOffset?: string | {
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        x?: number,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        y?: number
      },
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      data?: any,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      dragDirection?: 'both' | 'horizontal' | 'vertical',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      dragTemplate?: template | ((dragInfo: { itemData?: any, itemElement?: TElement }, containerElement: TElement) => string | TElement),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      dropFeedbackMode?: 'push' | 'indicate',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      filter?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      group?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      handle?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      onAdd?: ((e: { component?: T, event?: TEvent, itemData?: any, itemElement?: TElement, fromIndex?: number, toIndex?: number, fromComponent?: dxSortable | dxDraggable, toComponent?: dxSortable | dxDraggable, fromData?: any, toData?: any, dropInsideItem?: boolean }) => void),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      onDragChange?: ((e: { component?: T, event?: TEvent, cancel?: boolean, itemData?: any, itemElement?: TElement, fromIndex?: number, toIndex?: number, fromComponent?: dxSortable | dxDraggable, toComponent?: dxSortable | dxDraggable, fromData?: any, toData?: any, dropInsideItem?: boolean }) => void),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      onDragEnd?: ((e: { component?: T, event?: TEvent, cancel?: boolean, itemData?: any, itemElement?: TElement, fromIndex?: number, toIndex?: number, fromComponent?: dxSortable | dxDraggable, toComponent?: dxSortable | dxDraggable, fromData?: any, toData?: any, dropInsideItem?: boolean }) => void),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      onDragMove?: ((e: { component?: T, event?: TEvent, cancel?: boolean, itemData?: any, itemElement?: TElement, fromIndex?: number, toIndex?: number, fromComponent?: dxSortable | dxDraggable, toComponent?: dxSortable | dxDraggable, fromData?: any, toData?: any, dropInsideItem?: boolean }) => void),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      onDragStart?: ((e: { component?: T, event?: TEvent, cancel?: boolean, itemData?: any, itemElement?: TElement, fromIndex?: number, fromData?: any }) => void),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      onRemove?: ((e: { component?: T, event?: TEvent, itemData?: any, itemElement?: TElement, fromIndex?: number, toIndex?: number, fromComponent?: dxSortable | dxDraggable, toComponent?: dxSortable | dxDraggable, fromData?: any, toData?: any }) => void),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      onReorder?: ((e: { component?: T, event?: TEvent, itemData?: any, itemElement?: TElement, fromIndex?: number, toIndex?: number, fromComponent?: dxSortable | dxDraggable, toComponent?: dxSortable | dxDraggable, fromData?: any, toData?: any, dropInsideItem?: boolean, promise?: TPromise<void> }) => void),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      scrollSensitivity?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      scrollSpeed?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      showDragIcons?: boolean
    };
    /**
     */
    scrolling?: GridBaseScrolling;
    /**
     */
    searchPanel?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      highlightCaseSensitive?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      highlightSearchText?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      placeholder?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      searchVisibleColumnsOnly?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      text?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     */
    selectedRowKeys?: Array<any>;
    /**
     */
    selection?: GridBaseSelection;
    /**
     */
    showBorders?: boolean;
    /**
     */
    showColumnHeaders?: boolean;
    /**
     */
    showColumnLines?: boolean;
    /**
     */
    showRowLines?: boolean;
    /**
     */
    sorting?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      ascendingText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      clearText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      descendingText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      mode?: 'multiple' | 'none' | 'single',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      showSortIndexes?: boolean
    };
    /**
     */
    stateStoring?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      customLoad?: (() => TPromise<any>),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      customSave?: ((gridState: any) => any),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      enabled?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      savingTimeout?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      storageKey?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      type?: 'custom' | 'localStorage' | 'sessionStorage'
    };
    /**
     */
    twoWayBindingEnabled?: boolean;
    /**
     */
    wordWrapEnabled?: boolean;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface GridBaseEditing {
    /**
     * Specifies if confirmation is required when a user deletes a row.
     */
    confirmDelete?: boolean;
    /**
     * [tags] ctp Pending row changes.
     */
    changes?: Array<any>;
    /**
     * The name of a column being edited. Applies only if editing.mode is &apos;cell&apos; or &apos;batch&apos;.
     */
    editColumnName?: string;
    /**
     * The key(s) of a row being edited.
     */
    editRowKey?: any;
    /**
     * Configures the form. Used only if editing.mode is &apos;form&apos; or &apos;popup&apos;.
     */
    form?: dxFormOptions;
    /**
     * Specifies how a user edits data.
     */
    mode?: 'batch' | 'cell' | 'row' | 'form' | 'popup';
    /**
     * Configures the popup. Used only if editing.mode is &apos;popup&apos;.
     */
    popup?: dxPopupOptions;
    /**
     * Specifies operations that are performed after saving changes.
     */
    refreshMode?: 'full' | 'reshape' | 'repaint';
    /**
     * Specifies whether to select text in a cell when a user starts editing.
     */
    selectTextOnEditStart?: boolean;
    /**
     * Specifies whether a single or double click should switch a cell to the editing state. Applies if editing.mode is &apos;cell&apos; or &apos;batch&apos;.
     */
    startEditAction?: 'click' | 'dblClick';
    /**
     * Contains properties that specify texts for editing-related UI elements.
     */
    texts?: GridBaseEditingTexts;
    /**
     * Specifies whether the edit column uses icons instead of links.
     */
    useIcons?: boolean;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface GridBaseEditingTexts {
    /**
     * Specifies text for a hint that appears when a user pauses on the global &apos;Add&apos; button. Applies only if editing.allowAdding is true.
     */
    addRow?: string;
    /**
     * Specifies text for a hint that appears when a user pauses on the &apos;Discard&apos; button. Applies only if editing.mode is &apos;batch&apos;.
     */
    cancelAllChanges?: string;
    /**
     * Specifies text for a button that cancels changes in a row. Applies only if editing.allowUpdating is true and editing.mode is &apos;row&apos;.
     */
    cancelRowChanges?: string;
    /**
     * Specifies a message that prompts a user to confirm deletion.
     */
    confirmDeleteMessage?: string;
    /**
     * Specifies a title for the window that asks a user to confirm deletion.
     */
    confirmDeleteTitle?: string;
    /**
     * Specifies text for buttons that delete rows. Applies only if allowDeleting is true.
     */
    deleteRow?: string;
    /**
     * Specifies text for buttons that switch rows into the editing state. Applies only if allowUpdating is true.
     */
    editRow?: string;
    /**
     * Specifies text for a hint that appears when a user pauses on the global &apos;Save&apos; button. Applies only if editing.mode is &apos;batch&apos;.
     */
    saveAllChanges?: string;
    /**
     * Specifies text for a button that saves changes made in a row. Applies only if allowUpdating is true.
     */
    saveRowChanges?: string;
    /**
     * Specifies text for buttons that recover deleted rows. Applies only if allowDeleting is true and editing.mode is &apos;batch&apos;.
     */
    undeleteRow?: string;
    /**
     * Specifies text for a hint appearing when a user pauses on the button that cancels changes in a cell. Applies only if editing.mode is &apos;cell&apos; and data validation is enabled.
     */
    validationCancelChanges?: string;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface GridBasePaging {
    /**
     * Enables paging.
     */
    enabled?: boolean;
    /**
     * Specifies the page to be displayed using a zero-based index.
     */
    pageIndex?: number;
    /**
     * Specifies the page size.
     */
    pageSize?: number;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface GridBaseScrolling {
    /**
     * Specifies the rendering mode for columns. Applies when columns are left outside the viewport. Requires the columnWidth, columnAutoWidth, or width (for all columns) property specified.
     */
    columnRenderingMode?: 'standard' | 'virtual';
    /**
     * Specifies whether the UI component should load adjacent pages. Applies only if scrolling.mode is &apos;virtual&apos; or &apos;infinite&apos;.
     */
    preloadEnabled?: boolean;
    /**
     * Specifies the rendering mode for loaded rows.
     */
    rowRenderingMode?: 'standard' | 'virtual';
    /**
     * Specifies whether a user can scroll the content with a swipe gesture. Applies only if useNative is false.
     */
    scrollByContent?: boolean;
    /**
     * Specifies whether a user can scroll the content with the scrollbar. Applies only if useNative is false.
     */
    scrollByThumb?: boolean;
    /**
     * Specifies when to show scrollbars. Applies only if useNative is false.
     */
    showScrollbar?: 'always' | 'never' | 'onHover' | 'onScroll';
    /**
     * Specifies whether the UI component should use native or simulated scrolling.
     */
    useNative?: boolean | 'auto';
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface GridBaseSelection {
    /**
     * Specifies whether a user can select all rows at once.
     */
    allowSelectAll?: boolean;
    /**
     * Specifies the selection mode.
     */
    mode?: 'multiple' | 'none' | 'single';
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface GridBase {
    /**
     */
    beginCustomLoading(messageText: string): void;
    /**
     */
    byKey(key: any | string | number): TPromise<any>;
    /**
     */
    cancelEditData(): void;
    /**
     */
    cellValue(rowIndex: number, dataField: string): any;
    /**
     */
    cellValue(rowIndex: number, dataField: string, value: any): void;
    /**
     */
    cellValue(rowIndex: number, visibleColumnIndex: number): any;
    /**
     */
    cellValue(rowIndex: number, visibleColumnIndex: number, value: any): void;
    /**
     */
    clearFilter(): void;
    /**
     */
    clearFilter(filterName: string): void;
    /**
     */
    clearSelection(): void;
    /**
     */
    clearSorting(): void;
    /**
     */
    closeEditCell(): void;
    /**
     */
    collapseAdaptiveDetailRow(): void;
    /**
     */
    columnCount(): number;
    /**
     */
    columnOption(id: number | string): any;
    /**
     */
    columnOption(id: number | string, optionName: string): any;
    /**
     */
    columnOption(id: number | string, optionName: string, optionValue: any): void;
    /**
     */
    columnOption(id: number | string, options: any): void;
    /**
     */
    deleteColumn(id: number | string): void;
    /**
     */
    deleteRow(rowIndex: number): void;
    /**
     */
    deselectAll(): TPromise<void>;
    /**
     */
    deselectRows(keys: Array<any>): TPromise<any>;
    /**
     */
    editCell(rowIndex: number, dataField: string): void;
    /**
     */
    editCell(rowIndex: number, visibleColumnIndex: number): void;
    /**
     */
    editRow(rowIndex: number): void;
    /**
     */
    endCustomLoading(): void;
    /**
     */
    expandAdaptiveDetailRow(key: any): void;
    /**
     */
    filter(): any;
    /**
     */
    filter(filterExpr: any): void;
    focus(): void;
    /**
     */
    focus(element: TElement): void;
    /**
     */
    getCellElement(rowIndex: number, dataField: string): TElement | undefined;
    /**
     */
    getCellElement(rowIndex: number, visibleColumnIndex: number): TElement | undefined;
    /**
     */
    getCombinedFilter(): any;
    /**
     */
    getCombinedFilter(returnDataField: boolean): any;
    getDataSource(): DataSource;
    /**
     */
    getKeyByRowIndex(rowIndex: number): any;
    /**
     */
    getRowElement(rowIndex: number): TElementsArray | undefined;
    /**
     */
    getRowIndexByKey(key: any | string | number): number;
    /**
     */
    getScrollable(): dxScrollable;
    /**
     */
    getVisibleColumnIndex(id: number | string): number;
    /**
     */
    hasEditData(): boolean;
    /**
     */
    hideColumnChooser(): void;
    /**
     */
    isAdaptiveDetailRowExpanded(key: any): boolean;
    /**
     */
    isRowFocused(key: any): boolean;
    /**
     */
    isRowSelected(key: any): boolean;
    /**
     */
    keyOf(obj: any): any;
    /**
     */
    navigateToRow(key: any): void;
    /**
     */
    pageCount(): number;
    /**
     */
    pageIndex(): number;
    /**
     */
    pageIndex(newIndex: number): TPromise<void>;
    /**
     */
    pageSize(): number;
    /**
     */
    pageSize(value: number): void;
    /**
     */
    refresh(): TPromise<void>;
    /**
     */
    refresh(changesOnly: boolean): TPromise<void>;
    /**
     */
    repaintRows(rowIndexes: Array<number>): void;
    /**
     */
    saveEditData(): TPromise<void>;
    /**
     */
    searchByText(text: string): void;
    /**
     */
    selectAll(): TPromise<void>;
    /**
     */
    selectRows(keys: Array<any>, preserve: boolean): TPromise<any>;
    /**
     */
    selectRowsByIndexes(indexes: Array<number>): TPromise<any>;
    /**
     */
    showColumnChooser(): void;
    /**
     */
    state(): any;
    /**
     */
    state(state: any): void;
    /**
     */
    undeleteRow(rowIndex: number): void;
    /**
     */
    updateDimensions(): void;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface GridBaseColumn {
    /**
     */
    alignment?: 'center' | 'left' | 'right' | undefined;
    /**
     */
    allowEditing?: boolean;
    /**
     */
    allowFiltering?: boolean;
    /**
     */
    allowFixing?: boolean;
    /**
     */
    allowHeaderFiltering?: boolean;
    /**
     */
    allowHiding?: boolean;
    /**
     */
    allowReordering?: boolean;
    /**
     */
    allowResizing?: boolean;
    /**
     */
    allowSearch?: boolean;
    /**
     */
    allowSorting?: boolean;
    /**
     */
    calculateCellValue?: ((rowData: any) => any);
    /**
     */
    calculateDisplayValue?: string | ((rowData: any) => any);
    /**
     */
    calculateFilterExpression?: ((filterValue: any, selectedFilterOperation: string, target: string) => string | Array<any> | Function);
    /**
     */
    calculateSortValue?: string | ((rowData: any) => any);
    /**
     */
    caption?: string;
    /**
     */
    cssClass?: string;
    /**
     */
    customizeText?: ((cellInfo: { value?: string | number | Date, valueText?: string, target?: string, groupInterval?: string | number }) => string);
    /**
     */
    dataField?: string;
    /**
     */
    dataType?: 'string' | 'number' | 'date' | 'boolean' | 'object' | 'datetime';
    /**
     */
    editorOptions?: any;
    /**
     */
    encodeHtml?: boolean;
    /**
     */
    falseText?: string;
    /**
     */
    filterOperations?: Array<'=' | '<>' | '<' | '<=' | '>' | '>=' | 'contains' | 'endswith' | 'isblank' | 'isnotblank' | 'notcontains' | 'startswith' | 'between' | 'anyof' | 'noneof'>;
    /**
     */
    filterType?: 'exclude' | 'include';
    /**
     */
    filterValue?: any;
    /**
     */
    filterValues?: Array<any>;
    /**
     */
    fixed?: boolean;
    /**
     */
    fixedPosition?: 'left' | 'right';
    /**
     */
    formItem?: dxFormSimpleItem;
    /**
     */
    format?: format;
    /**
     */
    headerFilter?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      allowSearch?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      dataSource?: Array<any> | ((options: { component?: any, dataSource?: DataSourceOptions }) => any) | DataSourceOptions,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      groupInterval?: 'day' | 'hour' | 'minute' | 'month' | 'quarter' | 'second' | 'year' | number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      height?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      searchMode?: 'contains' | 'startswith' | 'equals',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     */
    hidingPriority?: number;
    /**
     */
    isBand?: boolean;
    /**
     */
    lookup?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      allowClearing?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      dataSource?: Array<any> | DataSourceOptions | Store | ((options: { data?: any, key?: any }) => Array<any> | DataSourceOptions | Store),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      displayExpr?: string | ((data: any) => string),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      valueExpr?: string
    };
    /**
     */
    minWidth?: number;
    /**
     */
    name?: string;
    /**
     */
    ownerBand?: number;
    /**
     */
    renderAsync?: boolean;
    /**
     */
    selectedFilterOperation?: '<' | '<=' | '<>' | '=' | '>' | '>=' | 'between' | 'contains' | 'endswith' | 'notcontains' | 'startswith';
    /**
     */
    setCellValue?: ((newData: any, value: any, currentRowData: any) => void | TPromise<void>);
    /**
     */
    showEditorAlways?: boolean;
    /**
     */
    showInColumnChooser?: boolean;
    /**
     */
    sortIndex?: number;
    /**
     */
    sortOrder?: 'asc' | 'desc' | undefined;
    /**
     */
    sortingMethod?: ((value1: any, value2: any) => number);
    /**
     */
    trueText?: string;
    /**
     */
    validationRules?: Array<RequiredRule | NumericRule | RangeRule | StringLengthRule | CustomRule | CompareRule | PatternRule | EmailRule | AsyncRule>;
    /**
     */
    visible?: boolean;
    /**
     */
    visibleIndex?: number;
    /**
     */
    width?: number | string;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface GridBaseColumnButton {
    /**
     */
    cssClass?: string;
    /**
     */
    hint?: string;
    /**
     */
    icon?: string;
    /**
     */
    text?: string;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxDataGridOptions extends GridBaseOptions<dxDataGrid> {
    /**
     */
    columns?: Array<dxDataGridColumn | string>;
    /**
     */
    customizeColumns?: ((columns: Array<dxDataGridColumn>) => any);
    /**
     * @deprecated
     */
    customizeExportData?: ((columns: Array<dxDataGridColumn>, rows: Array<dxDataGridRowObject>) => any);
    /**
     */
    editing?: dxDataGridEditing;
    /**
     */
    export?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      allowExportSelectedData?: boolean,
      /**
       * @deprecated
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      customizeExcelCell?: ((options: { component?: dxDataGrid, horizontalAlignment?: 'center' | 'centerContinuous' | 'distributed' | 'fill' | 'general' | 'justify' | 'left' | 'right', verticalAlignment?: 'bottom' | 'center' | 'distributed' | 'justify' | 'top', wrapTextEnabled?: boolean, backgroundColor?: string, fillPatternType?: 'darkDown' | 'darkGray' | 'darkGrid' | 'darkHorizontal' | 'darkTrellis' | 'darkUp' | 'darkVertical' | 'gray0625' | 'gray125' | 'lightDown' | 'lightGray' | 'lightGrid' | 'lightHorizontal' | 'lightTrellis' | 'lightUp' | 'lightVertical' | 'mediumGray' | 'none' | 'solid', fillPatternColor?: string, font?: ExcelFont, value?: string | number | Date, numberFormat?: string, gridCell?: ExcelDataGridCell }) => any),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      enabled?: boolean,
      /**
       * @deprecated
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      excelFilterEnabled?: boolean,
      /**
       * @deprecated
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      excelWrapTextEnabled?: boolean,
      /**
       * @deprecated
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      fileName?: string,
      /**
       * @deprecated
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      ignoreExcelErrors?: boolean,
      /**
       * @deprecated
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      proxyUrl?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      texts?: {
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        exportAll?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        exportSelectedRows?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        exportTo?: string
      }
    };
    /**
     */
    groupPanel?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      allowColumnDragging?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      emptyPanelText?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean | 'auto'
    };
    /**
     */
    grouping?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      allowCollapsing?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      autoExpandAll?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      contextMenuEnabled?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      expandMode?: 'buttonClick' | 'rowClick',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      texts?: {
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        groupByThisColumn?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        groupContinuedMessage?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        groupContinuesMessage?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        ungroup?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        ungroupAll?: string
      }
    };
    /**
     */
    keyExpr?: string | Array<string>;
    /**
     */
    masterDetail?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      autoExpandAll?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      enabled?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      template?: template | ((detailElement: TElement, detailInfo: { key?: any, data?: any, watch?: Function }) => any)
    };
    /**
     */
    onCellClick?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, event?: TEvent, data?: any, key?: any, value?: any, displayValue?: any, text?: string, columnIndex?: number, column?: any, rowIndex?: number, rowType?: string, cellElement?: TElement, row?: dxDataGridRowObject }) => void) | string;
    /**
     */
    onCellDblClick?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, event?: TEvent, data?: any, key?: any, value?: any, displayValue?: any, text?: string, columnIndex?: number, column?: dxDataGridColumn, rowIndex?: number, rowType?: string, cellElement?: TElement, row?: dxDataGridRowObject }) => void);
    /**
     */
    onCellHoverChanged?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, eventType?: string, data?: any, key?: any, value?: any, text?: string, displayValue?: any, columnIndex?: number, rowIndex?: number, column?: dxDataGridColumn, rowType?: string, cellElement?: TElement, row?: dxDataGridRowObject }) => void);
    /**
     */
    onCellPrepared?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, data?: any, key?: any, value?: any, displayValue?: any, text?: string, columnIndex?: number, column?: dxDataGridColumn, rowIndex?: number, rowType?: string, row?: dxDataGridRowObject, isSelected?: boolean, isExpanded?: boolean, isNewRow?: boolean, cellElement?: TElement, watch?: Function, oldValue?: any }) => void);
    /**
     */
    onContextMenuPreparing?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, items?: Array<any>, target?: string, targetElement?: TElement, columnIndex?: number, column?: dxDataGridColumn, rowIndex?: number, row?: dxDataGridRowObject }) => void);
    /**
     */
    onEditingStart?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, data?: any, key?: any, cancel?: boolean, column?: any }) => void);
    /**
     */
    onEditorPrepared?: ((options: { component?: dxDataGrid, element?: TElement, model?: any, parentType?: string, value?: any, setValue?: any, updateValueTimeout?: number, width?: number, disabled?: boolean, rtlEnabled?: boolean, editorElement?: TElement, readOnly?: boolean, dataField?: string, row?: dxDataGridRowObject }) => void);
    /**
     */
    onEditorPreparing?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, parentType?: string, value?: any, setValue?: any, updateValueTimeout?: number, width?: number, disabled?: boolean, rtlEnabled?: boolean, cancel?: boolean, editorElement?: TElement, readOnly?: boolean, editorName?: string, editorOptions?: any, dataField?: string, row?: dxDataGridRowObject }) => void);
    /**
     * @deprecated
     */
    onExported?: ((e: { component?: dxDataGrid, element?: TElement, model?: any }) => void);
    /**
     */
    onExporting?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, fileName?: string, cancel?: boolean }) => void);
    /**
     * @deprecated
     */
    onFileSaving?: ((e: { component?: dxDataGrid, element?: TElement, fileName?: string, format?: string, data?: Blob, cancel?: boolean }) => void);
    /**
     */
    onFocusedCellChanged?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, cellElement?: TElement, columnIndex?: number, rowIndex?: number, row?: dxDataGridRowObject, column?: dxDataGridColumn }) => void);
    /**
     */
    onFocusedCellChanging?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, cellElement?: TElement, prevColumnIndex?: number, prevRowIndex?: number, newColumnIndex?: number, newRowIndex?: number, event?: TEvent, rows?: Array<dxDataGridRowObject>, columns?: Array<dxDataGridColumn>, cancel?: boolean, isHighlighted?: boolean }) => void);
    /**
     */
    onFocusedRowChanged?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, rowElement?: TElement, rowIndex?: number, row?: dxDataGridRowObject }) => void);
    /**
     */
    onFocusedRowChanging?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, rowElement?: TElement, prevRowIndex?: number, newRowIndex?: number, event?: TEvent, rows?: Array<dxDataGridRowObject>, cancel?: boolean }) => void);
    /**
     */
    onRowClick?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, event?: TEvent, data?: any, key?: any, values?: Array<any>, columns?: Array<any>, rowIndex?: number, rowType?: string, isSelected?: boolean, isExpanded?: boolean, isNewRow?: boolean, groupIndex?: number, rowElement?: TElement, handled?: boolean }) => void) | string;
    /**
     */
    onRowDblClick?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, event?: TEvent, data?: any, key?: any, values?: Array<any>, columns?: Array<dxDataGridColumn>, rowIndex?: number, rowType?: string, isSelected?: boolean, isExpanded?: boolean, isNewRow?: boolean, groupIndex?: number, rowElement?: TElement }) => void);
    /**
     */
    onRowPrepared?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, data?: any, key?: any, values?: Array<any>, columns?: Array<dxDataGridColumn>, rowIndex?: number, rowType?: string, groupIndex?: number, isSelected?: boolean, isExpanded?: boolean, isNewRow?: boolean, rowElement?: TElement }) => void);
    /**
     */
    remoteOperations?: boolean | {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      filtering?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      groupPaging?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      grouping?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      paging?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      sorting?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      summary?: boolean
    } | 'auto';
    /**
     */
    rowTemplate?: template | ((rowElement: TElement, rowInfo: { key?: any, data?: any, component?: dxDataGrid, values?: Array<any>, rowIndex?: number, columns?: Array<dxDataGridColumn>, isSelected?: boolean, rowType?: string, groupIndex?: number, isExpanded?: boolean }) => any);
    /**
     */
    scrolling?: dxDataGridScrolling;
    /**
     */
    selection?: dxDataGridSelection;
    /**
     */
    selectionFilter?: string | Array<any> | Function;
    /**
     */
    sortByGroupSummaryInfo?: Array<{
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      groupColumn?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      sortOrder?: 'asc' | 'desc',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      summaryItem?: string | number
    }>;
    /**
     */
    summary?: {
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      calculateCustomSummary?: ((options: { component?: dxDataGrid, name?: string, summaryProcess?: string, value?: any, totalValue?: any, groupIndex?: number }) => any),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      groupItems?: Array<{
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        alignByColumn?: boolean,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        column?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        customizeText?: ((itemInfo: { value?: string | number | Date, valueText?: string }) => string),
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        displayFormat?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        name?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        showInColumn?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        showInGroupFooter?: boolean,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        skipEmptyValues?: boolean,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        summaryType?: 'avg' | 'count' | 'custom' | 'max' | 'min' | 'sum' | string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        valueFormat?: format
      }>,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      recalculateWhileEditing?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      skipEmptyValues?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      texts?: {
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        avg?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        avgOtherColumn?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        count?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        max?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        maxOtherColumn?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        min?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        minOtherColumn?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        sum?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        sumOtherColumn?: string
      },
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      totalItems?: Array<{
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        alignment?: 'center' | 'left' | 'right',
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        column?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        cssClass?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        customizeText?: ((itemInfo: { value?: string | number | Date, valueText?: string }) => string),
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        displayFormat?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        name?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        showInColumn?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        skipEmptyValues?: boolean,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        summaryType?: 'avg' | 'count' | 'custom' | 'max' | 'min' | 'sum' | string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        valueFormat?: format
      }>
    };
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxDataGridEditing extends GridBaseEditing {
    /**
     * Specifies whether a user can add new rows.
     */
    allowAdding?: boolean;
    /**
     * Specifies whether a user can delete rows. It is called for each data row when defined as a function.
     */
    allowDeleting?: boolean | ((options: { component?: dxDataGrid, row?: dxDataGridRowObject }) => boolean);
    /**
     * Specifies whether a user can update rows. It is called for each data row when defined as a function.
     */
    allowUpdating?: boolean | ((options: { component?: dxDataGrid, row?: dxDataGridRowObject }) => boolean);
    /**
     * 
     */
    texts?: any;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxDataGridScrolling extends GridBaseScrolling {
    /**
     * Specifies the scrolling mode.
     */
    mode?: 'infinite' | 'standard' | 'virtual';
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxDataGridSelection extends GridBaseSelection {
    /**
     * Makes selection deferred.
     */
    deferred?: boolean;
    /**
     * Specifies the mode in which all the records are selected. Applies only if selection.allowSelectAll is true.
     */
    selectAllMode?: 'allPages' | 'page';
    /**
     * Specifies when to display the selection column and row selection checkboxes. Applies only if selection.mode is &apos;multiple&apos;.
     */
    showCheckBoxesMode?: 'always' | 'none' | 'onClick' | 'onLongTap';
}
/**
 */
declare class dxDataGrid extends Widget implements GridBase {
    constructor(element: TElement, options?: dxDataGridOptions)
    /**
     */
    addColumn(columnOptions: any | string): void;
    /**
     */
    addRow(): TPromise<void>;
    /**
     */
    clearGrouping(): void;
    /**
     */
    collapseAll(groupIndex?: number): void;
    /**
     */
    collapseRow(key: any): TPromise<void>;
    /**
     */
    expandAll(groupIndex?: number): void;
    /**
     */
    expandRow(key: any): TPromise<void>;
    /**
     * @deprecated
     */
    exportToExcel(selectionOnly: boolean): void;
    /**
     */
    getSelectedRowKeys(): Array<any> & TPromise<any>;
    /**
     */
    getSelectedRowsData(): Array<any> & TPromise<any>;
    /**
     */
    getTotalSummaryValue(summaryItemName: string): any;
    /**
     */
    getVisibleColumns(): Array<dxDataGridColumn>;
    /**
     */
    getVisibleColumns(headerLevel: number): Array<dxDataGridColumn>;
    /**
     */
    getVisibleRows(): Array<dxDataGridRowObject>;
    /**
     */
    isRowExpanded(key: any): boolean;
    /**
     */
    isRowSelected(data: any): boolean;
    isRowSelected(key: any): boolean;
    /**
     */
    totalCount(): number;

    beginCustomLoading(messageText: string): void;
    byKey(key: any | string | number): TPromise<any>;
    cancelEditData(): void;
    cellValue(rowIndex: number, dataField: string): any;
    cellValue(rowIndex: number, dataField: string, value: any): void;
    cellValue(rowIndex: number, visibleColumnIndex: number): any;
    cellValue(rowIndex: number, visibleColumnIndex: number, value: any): void;
    clearFilter(): void;
    clearFilter(filterName: string): void;
    clearSelection(): void;
    clearSorting(): void;
    closeEditCell(): void;
    collapseAdaptiveDetailRow(): void;
    columnCount(): number;
    columnOption(id: number | string): any;
    columnOption(id: number | string, optionName: string): any;
    columnOption(id: number | string, optionName: string, optionValue: any): void;
    columnOption(id: number | string, options: any): void;
    deleteColumn(id: number | string): void;
    deleteRow(rowIndex: number): void;
    deselectAll(): TPromise<void>;
    deselectRows(keys: Array<any>): TPromise<any>;
    editCell(rowIndex: number, dataField: string): void;
    editCell(rowIndex: number, visibleColumnIndex: number): void;
    editRow(rowIndex: number): void;
    endCustomLoading(): void;
    expandAdaptiveDetailRow(key: any): void;
    filter(): any;
    filter(filterExpr: any): void;
    focus(): void;
    focus(element: TElement): void;
    getCellElement(rowIndex: number, dataField: string): TElement | undefined;
    getCellElement(rowIndex: number, visibleColumnIndex: number): TElement | undefined;
    getCombinedFilter(): any;
    getCombinedFilter(returnDataField: boolean): any;
    getDataSource(): DataSource;
    getKeyByRowIndex(rowIndex: number): any;
    getRowElement(rowIndex: number): TElementsArray | undefined;
    getRowIndexByKey(key: any | string | number): number;
    getScrollable(): dxScrollable;
    getVisibleColumnIndex(id: number | string): number;
    hasEditData(): boolean;
    hideColumnChooser(): void;
    isAdaptiveDetailRowExpanded(key: any): boolean;
    isRowFocused(key: any): boolean;
    isRowSelected(key: any): boolean;
    keyOf(obj: any): any;
    navigateToRow(key: any): void;
    pageCount(): number;
    pageIndex(): number;
    pageIndex(newIndex: number): TPromise<void>;
    pageSize(): number;
    pageSize(value: number): void;
    refresh(): TPromise<void>;
    refresh(changesOnly: boolean): TPromise<void>;
    repaintRows(rowIndexes: Array<number>): void;
    saveEditData(): TPromise<void>;
    searchByText(text: string): void;
    selectAll(): TPromise<void>;
    selectRows(keys: Array<any>, preserve: boolean): TPromise<any>;
    selectRowsByIndexes(indexes: Array<number>): TPromise<any>;
    showColumnChooser(): void;
    state(): any;
    state(state: any): void;
    undeleteRow(rowIndex: number): void;
    updateDimensions(): void;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxDataGridColumn extends GridBaseColumn {
    /**
     */
    allowExporting?: boolean;
    /**
     */
    allowGrouping?: boolean;
    /**
     */
    autoExpandGroup?: boolean;
    /**
     */
    buttons?: Array<'cancel' | 'delete' | 'edit' | 'save' | 'undelete' | dxDataGridColumnButton>;
    /**
     */
    calculateGroupValue?: string | ((rowData: any) => any);
    /**
     */
    cellTemplate?: template | ((cellElement: TElement, cellInfo: { data?: any, component?: dxDataGrid, value?: any, oldValue?: any, displayValue?: any, text?: string, columnIndex?: number, rowIndex?: number, column?: dxDataGridColumn, row?: dxDataGridRowObject, rowType?: string, watch?: Function }) => any);
    /**
     */
    columns?: Array<dxDataGridColumn | string>;
    /**
     */
    editCellTemplate?: template | ((cellElement: TElement, cellInfo: { setValue?: any, data?: any, component?: dxDataGrid, value?: any, displayValue?: any, text?: string, columnIndex?: number, rowIndex?: number, column?: dxDataGridColumn, row?: dxDataGridRowObject, rowType?: string, watch?: Function }) => any);
    /**
     */
    groupCellTemplate?: template | ((cellElement: TElement, cellInfo: { data?: any, component?: dxDataGrid, value?: any, text?: string, displayValue?: any, columnIndex?: number, rowIndex?: number, column?: dxDataGridColumn, row?: dxDataGridRowObject, summaryItems?: Array<any>, groupContinuesMessage?: string, groupContinuedMessage?: string }) => any);
    /**
     */
    groupIndex?: number;
    /**
     */
    headerCellTemplate?: template | ((columnHeader: TElement, headerInfo: { component?: dxDataGrid, columnIndex?: number, column?: dxDataGridColumn }) => any);
    /**
     */
    showWhenGrouped?: boolean;
    /**
     */
    type?: 'adaptive' | 'buttons' | 'detailExpand' | 'groupExpand' | 'selection';
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxDataGridColumnButton extends GridBaseColumnButton {
    /**
     */
    name?: 'cancel' | 'delete' | 'edit' | 'save' | 'undelete' | string;
    /**
     */
    onClick?: ((e: { component?: dxDataGrid, element?: TElement, model?: any, event?: TEvent, row?: dxDataGridRowObject, column?: dxDataGridColumn }) => void) | string;
    /**
     */
    template?: template | ((cellElement: TElement, cellInfo: { component?: dxDataGrid, data?: any, key?: any, columnIndex?: number, column?: dxDataGridColumn, rowIndex?: number, rowType?: string, row?: dxDataGridRowObject }) => string | TElement);
    /**
     */
    visible?: boolean | ((options: { component?: dxDataGrid, row?: dxDataGridRowObject, column?: dxDataGridColumn }) => boolean);
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxDataGridRowObject {
    /**
     */
    data?: any;
    /**
     */
    groupIndex?: number;
    /**
     */
    isEditing?: boolean;
    /**
     */
    isExpanded?: boolean;
    /**
     */
    isNewRow?: boolean;
    /**
     */
    isSelected?: boolean;
    /**
     */
    key?: any;
    /**
     */
    rowIndex?: number;
    /**
     */
    rowType?: string;
    /**
     */
    values?: Array<any>;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxDataGridOptions;

/** @deprecated use Options instead */
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxDataGridOptions;
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Column = dxDataGridColumn;

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export default dxDataGrid;
