/**
* DevExtreme (ui/file_uploader.d.ts)
* Version: 21.1.0 (build 21089-0329)
* Build date: Tue Mar 30 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import {
    TEvent
} from '../events/index';

import Editor, {
    EditorOptions
} from './editor/editor';

import UploadInfo from '../file_management/upload_info';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxFileUploaderOptions extends EditorOptions<dxFileUploader> {
    /**
     */
    abortUpload?: ((file: File, uploadInfo?: UploadInfo) => TPromise<any> | any);
    /**
     */
    accept?: string;
    /**
     */
    allowCanceling?: boolean;
    /**
     */
    allowedFileExtensions?: Array<string>;
    /**
     */
    chunkSize?: number;
    /**
     */
    focusStateEnabled?: boolean;
    /**
     */
    invalidFileExtensionMessage?: string;
    /**
     */
    invalidMaxFileSizeMessage?: string;
    /**
     */
    invalidMinFileSizeMessage?: string;
    /**
     */
    inputAttr?: any;
    /**
     */
    labelText?: string;
    /**
     */
    maxFileSize?: number;
    /**
     */
    minFileSize?: number;
    /**
     */
    multiple?: boolean;
    /**
     */
    name?: string;
    /**
     */
    onBeforeSend?: ((e: { component?: dxFileUploader, element?: TElement, model?: any, request?: XMLHttpRequest, file?: File, uploadInfo?: UploadInfo }) => void);
    /**
     */
    onDropZoneEnter?: ((e: { component?: dxFileUploader, element?: TElement, model?: any, dropZoneElement?: TElement, event?: TEvent }) => void);
    /**
     */
    onDropZoneLeave?: ((e: { component?: dxFileUploader, element?: TElement, model?: any, dropZoneElement?: TElement, event?: TEvent }) => void);
    /**
     */
    onFilesUploaded?: ((e: { component?: dxFileUploader, element?: TElement, model?: any }) => void);
    /**
     */
    onProgress?: ((e: { component?: dxFileUploader, element?: TElement, model?: any, file?: File, segmentSize?: number, bytesLoaded?: number, bytesTotal?: number, event?: TEvent, request?: XMLHttpRequest }) => void);
    /**
     */
    onUploadAborted?: ((e: { component?: dxFileUploader, element?: TElement, model?: any, file?: File, event?: TEvent, request?: XMLHttpRequest, message?: string }) => void);
    /**
     */
    onUploadError?: ((e: { component?: dxFileUploader, element?: TElement, model?: any, file?: File, event?: TEvent, request?: XMLHttpRequest, error?: any, message?: string }) => void);
    /**
     */
    onUploadStarted?: ((e: { component?: dxFileUploader, element?: TElement, model?: any, file?: File, event?: TEvent, request?: XMLHttpRequest }) => void);
    /**
     */
    onUploaded?: ((e: { component?: dxFileUploader, element?: TElement, model?: any, file?: File, event?: TEvent, request?: XMLHttpRequest, message?: string }) => void);
    /**
     */
    onValueChanged?: ((e: { component?: dxFileUploader, element?: TElement, model?: any, value?: Array<File>, previousValue?: Array<File>, event?: TEvent }) => void);
    /**
     */
    progress?: number;
    /**
     */
    readyToUploadMessage?: string;
    /**
     */
    selectButtonText?: string;
    /**
     */
    showFileList?: boolean;
    /**
     */
    dialogTrigger?: string | TElement;
    /**
     */
    dropZone?: string | TElement;
    /**
     */
    uploadButtonText?: string;
    /**
     */
    uploadChunk?: ((file: File, uploadInfo: UploadInfo) => TPromise<any> | any);
    /**
     */
    uploadFailedMessage?: string;
    /**
     */
    uploadAbortedMessage?: string;
    /**
     */
    uploadFile?: ((file: File, progressCallback: Function) => TPromise<any> | any);
    /**
     */
    uploadHeaders?: any;
    /**
     */
    uploadCustomData?: any;
    /**
     */
    uploadMethod?: 'POST' | 'PUT';
    /**
     */
    uploadMode?: 'instantly' | 'useButtons' | 'useForm';
    /**
     */
    uploadUrl?: string;
    /**
     */
    uploadedMessage?: string;
    /**
     */
    value?: Array<File>;
}
/**
 */
export default class dxFileUploader extends Editor {
    constructor(element: TElement, options?: dxFileUploaderOptions)
    /**
     */
    upload(): void;
    /**
     */
    upload(fileIndex: number): void;
    /**
     */
    upload(file: File): void;
    /**
     */
    abortUpload(): void;
    /**
     */
    abortUpload(fileIndex: number): void;
    /**
     */
    abortUpload(file: File): void;
    /**
     */
    removeFile(fileIndex: number): void;
    /**
     */
    removeFile(file: File): void;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxFileUploaderOptions;

/** @deprecated use Options instead */
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxFileUploaderOptions;
