/**
* DevExtreme (ui/list.d.ts)
* Version: 21.1.0 (build 21089-0329)
* Build date: Tue Mar 30 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    template
} from '../core/templates/template';

import {
    TPromise
} from '../core/utils/deferred';

import DataSource, {
    DataSourceOptions
} from '../data/data_source';

import {
    TEvent
} from '../events/index';

import CollectionWidget, {
    CollectionWidgetItem,
    CollectionWidgetOptions
} from './collection/ui.collection_widget.base';

import {
    dxSortableOptions
} from './sortable';

import {
    SearchBoxMixinOptions
} from './widget/ui.search_box_mixin';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxListOptions extends CollectionWidgetOptions<dxList>, SearchBoxMixinOptions<dxList> {
    /**
     */
    activeStateEnabled?: boolean;
    /**
     */
    allowItemDeleting?: boolean;
    /**
     */
    bounceEnabled?: boolean;
    /**
     */
    collapsibleGroups?: boolean;
    /**
     */
    dataSource?: string | Array<string | dxListItem | any> | DataSource | DataSourceOptions;
    /**
     */
    displayExpr?: string | ((item: any) => string);
    /**
     */
    focusStateEnabled?: boolean;
    /**
     */
    groupTemplate?: template | ((groupData: any, groupIndex: number, groupElement: TElement) => string | TElement);
    /**
     */
    grouped?: boolean;
    /**
     */
    hoverStateEnabled?: boolean;
    /**
     */
    indicateLoading?: boolean;
    /**
     */
    itemDeleteMode?: 'context' | 'slideButton' | 'slideItem' | 'static' | 'swipe' | 'toggle';
    /**
     */
    itemDragging?: dxSortableOptions;
    /**
     */
    items?: Array<string | dxListItem | any>;
    /**
     */
    menuItems?: Array<{
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      action?: ((itemElement: TElement, itemData: any) => any),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      text?: string
    }>;
    /**
     */
    menuMode?: 'context' | 'slide';
    /**
     */
    nextButtonText?: string;
    /**
     */
    onGroupRendered?: ((e: { component?: dxList, element?: TElement, model?: any, groupData?: any, groupElement?: TElement, groupIndex?: number }) => void);
    /**
     */
    onItemClick?: ((e: { component?: dxList, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number | any, event?: TEvent }) => void) | string;
    /**
     */
    onItemContextMenu?: ((e: { component?: dxList, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number | any, event?: TEvent }) => void);
    /**
     */
    onItemDeleted?: ((e: { component?: dxList, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number | any }) => void);
    /**
     */
    onItemDeleting?: ((e: { component?: dxList, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number | any, cancel?: boolean | TPromise<void> }) => void);
    /**
     */
    onItemHold?: ((e: { component?: dxList, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number | any, event?: TEvent }) => void);
    /**
     */
    onItemReordered?: ((e: { component?: dxList, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number | any, fromIndex?: number, toIndex?: number }) => void);
    /**
     */
    onItemSwipe?: ((e: { component?: dxList, element?: TElement, model?: any, event?: TEvent, itemData?: any, itemElement?: TElement, itemIndex?: number | any, direction?: string }) => void);
    /**
     */
    onPageLoading?: ((e: { component?: dxList, element?: TElement, model?: any }) => void);
    /**
     */
    onPullRefresh?: ((e: { component?: dxList, element?: TElement, model?: any }) => void);
    /**
     */
    onScroll?: ((e: { component?: dxList, element?: TElement, model?: any, event?: TEvent, scrollOffset?: any, reachedLeft?: boolean, reachedRight?: boolean, reachedTop?: boolean, reachedBottom?: boolean }) => void);
    /**
     */
    onSelectAllValueChanged?: ((e: { component?: dxList, element?: TElement, model?: any, value?: boolean }) => void);
    /**
     */
    pageLoadMode?: 'nextButton' | 'scrollBottom';
    /**
     */
    pageLoadingText?: string;
    /**
     */
    pullRefreshEnabled?: boolean;
    /**
     */
    pulledDownText?: string;
    /**
     */
    pullingDownText?: string;
    /**
     */
    refreshingText?: string;
    /**
     */
    repaintChangesOnly?: boolean;
    /**
     */
    scrollByContent?: boolean;
    /**
     */
    scrollByThumb?: boolean;
    /**
     */
    scrollingEnabled?: boolean;
    /**
     */
    selectAllMode?: 'allPages' | 'page';
    /**
     */
    selectionMode?: 'all' | 'multiple' | 'none' | 'single';
    /**
     */
    showScrollbar?: 'always' | 'never' | 'onHover' | 'onScroll';
    /**
     */
    showSelectionControls?: boolean;
    /**
     */
    useNativeScrolling?: boolean;
}
/**
 */
export default class dxList extends CollectionWidget {
    constructor(element: TElement, options?: dxListOptions)
    /**
     */
    clientHeight(): number;
    /**
     */
    collapseGroup(groupIndex: number): TPromise<void>;
    /**
     */
    deleteItem(itemElement: Element): TPromise<void>;
    /**
     */
    deleteItem(itemIndex: number | any): TPromise<void>;
    /**
     */
    expandGroup(groupIndex: number): TPromise<void>;
    /**
     */
    isItemSelected(itemElement: Element): boolean;
    /**
     */
    isItemSelected(itemIndex: number | any): boolean;
    /**
     */
    reload(): void;
    /**
     */
    reorderItem(itemElement: Element, toItemElement: Element): TPromise<void>;
    /**
     */
    reorderItem(itemIndex: number | any, toItemIndex: number | any): TPromise<void>;
    /**
     */
    scrollBy(distance: number): void;
    /**
     */
    scrollHeight(): number;
    /**
     */
    scrollTo(location: number): void;
    /**
     */
    scrollToItem(itemElement: Element): void;
    /**
     */
    scrollToItem(itemIndex: number | any): void;
    /**
     */
    scrollTop(): number;
    /**
     */
    selectAll(): void;
    /**
     */
    selectItem(itemElement: Element): void;
    /**
     */
    selectItem(itemIndex: number | any): void;
    /**
     */
    unselectAll(): void;
    /**
     */
    unselectItem(itemElement: Element): void;
    /**
     */
    unselectItem(itemIndex: number | any): void;
    /**
     */
    updateDimensions(): TPromise<void>;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxListItem extends CollectionWidgetItem {
    /**
     */
    badge?: string;
    /**
     */
    icon?: string;
    /**
     */
    key?: string;
    /**
     */
    showChevron?: boolean;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxListOptions;

/** @deprecated use Options instead */
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxListOptions;
