/**
* DevExtreme (ui/popup.d.ts)
* Version: 21.1.0 (build 21089-0329)
* Build date: Tue Mar 30 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    animationConfig
} from '../animation/fx';

import {
    positionConfig
} from '../animation/position';

import {
    TElement
} from '../core/element';

import {
    template
} from '../core/templates/template';

import dxOverlay, {
    dxOverlayAnimation,
    dxOverlayOptions
} from './overlay';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPopupOptions<T = dxPopup> extends dxOverlayOptions<T> {
    /**
     */
    animation?: dxPopupAnimation;
    /**
     */
    container?: string | TElement;
    /**
     */
    dragEnabled?: boolean;
    /**
     */
    focusStateEnabled?: boolean;
    /**
     */
    fullScreen?: boolean;
    /**
     */
    height?: number | string | (() => number | string);
    /**
     */
    onResize?: ((e: { component?: T, element?: TElement, model?: any }) => void);
    /**
     */
    onResizeEnd?: ((e: { component?: T, element?: TElement, model?: any }) => void);
    /**
     */
    onResizeStart?: ((e: { component?: T, element?: TElement, model?: any }) => void);
    /**
     */
    onTitleRendered?: ((e: { component?: T, element?: TElement, model?: any, titleElement?: TElement }) => void);
    /**
     */
    position?: 'bottom' | 'center' | 'left' | 'left bottom' | 'left top' | 'right' | 'right bottom' | 'right top' | 'top' | positionConfig | Function;
    /**
     */
    resizeEnabled?: boolean;
    /**
     */
    showCloseButton?: boolean;
    /**
     */
    showTitle?: boolean;
    /**
     */
    title?: string;
    /**
     */
    titleTemplate?: template | ((titleElement: TElement) => string | TElement);
    /**
     */
    toolbarItems?: Array<dxPopupToolbarItem>;
    /**
     */
    width?: number | string | (() => number | string);
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPopupAnimation extends dxOverlayAnimation {
    /**
     * 
     */
    hide?: animationConfig;
    /**
     * 
     */
    show?: animationConfig;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPopupToolbarItem {
    /**
     * Specifies whether the toolbar item responds to user interaction.
     */
    disabled?: boolean;
    /**
     * Specifies html code inserted into the toolbar item element.
     */
    html?: string;
    /**
     * Specifies a location for the item on the toolbar.
     */
    location?: 'after' | 'before' | 'center';
    /**
     * Configures the DevExtreme UI component used as a toolbar item.
     */
    options?: any;
    /**
     * Specifies a template that should be used to render this item only.
     */
    template?: template;
    /**
     * Specifies text displayed for the toolbar item.
     */
    text?: string;
    /**
     * Specifies whether the item is displayed on a top or bottom toolbar.
     */
    toolbar?: 'bottom' | 'top';
    /**
     * Specifies whether or not a UI component item must be displayed.
     */
    visible?: boolean;
    /**
     * A UI component that presents a toolbar item.
     */
    widget?: 'dxAutocomplete' | 'dxButton' | 'dxCheckBox' | 'dxDateBox' | 'dxMenu' | 'dxSelectBox' | 'dxTabs' | 'dxTextBox' | 'dxButtonGroup' | 'dxDropDownButton';
}
/**
 */
export default class dxPopup extends dxOverlay {
    constructor(element: TElement, options?: dxPopupOptions)
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxPopupOptions;

/** @deprecated use Options instead */
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxPopupOptions;
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type ToolbarItem = dxPopupToolbarItem;
