/**
* DevExtreme (ui/tree_view.d.ts)
* Version: 21.1.0 (build 21089-0329)
* Build date: Tue Mar 30 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import DataSource, {
    DataSourceOptions
} from '../data/data_source';

import {
    TEvent
} from '../events/index';

import {
    CollectionWidgetItem
} from './collection/ui.collection_widget.base';

import HierarchicalCollectionWidget, {
    HierarchicalCollectionWidgetOptions
} from './hierarchical_collection/ui.hierarchical_collection_widget';

import {
    SearchBoxMixinOptions
} from './widget/ui.search_box_mixin';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxTreeViewOptions extends HierarchicalCollectionWidgetOptions<dxTreeView>, SearchBoxMixinOptions<dxTreeView> {
    /**
     */
    animationEnabled?: boolean;
    /**
     */
    createChildren?: ((parentNode: dxTreeViewNode) => TPromise<any> | Array<any>);
    /**
     */
    dataSource?: string | Array<dxTreeViewItem> | DataSource | DataSourceOptions;
    /**
     */
    dataStructure?: 'plain' | 'tree';
    /**
     */
    expandAllEnabled?: boolean;
    /**
     */
    expandEvent?: 'dblclick' | 'click';
    /**
     */
    expandNodesRecursive?: boolean;
    /**
     */
    expandedExpr?: string | Function;
    /**
     */
    hasItemsExpr?: string | Function;
    /**
     */
    items?: Array<dxTreeViewItem>;
    /**
     */
    onItemClick?: ((e: { component?: dxTreeView, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number | any, event?: TEvent, node?: dxTreeViewNode }) => void);
    /**
     */
    onItemCollapsed?: ((e: { component?: dxTreeView, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number, event?: TEvent, node?: dxTreeViewNode }) => void);
    /**
     */
    onItemContextMenu?: ((e: { component?: dxTreeView, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number | any, event?: TEvent, node?: dxTreeViewNode }) => void);
    /**
     */
    onItemExpanded?: ((e: { component?: dxTreeView, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number, event?: TEvent, node?: dxTreeViewNode }) => void);
    /**
     */
    onItemHold?: ((e: { component?: dxTreeView, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number, event?: TEvent, node?: dxTreeViewNode }) => void);
    /**
     */
    onItemRendered?: ((e: { component?: dxTreeView, element?: TElement, model?: any, itemData?: any, itemElement?: TElement, itemIndex?: number, node?: dxTreeViewNode }) => void);
    /**
     */
    onItemSelectionChanged?: ((e: { component?: dxTreeView, element?: TElement, model?: any, node?: dxTreeViewNode, itemElement?: TElement }) => void);
    /**
     */
    onSelectAllValueChanged?: ((e: { component?: dxTreeView, element?: TElement, model?: any, value?: boolean }) => void);
    /**
     */
    onSelectionChanged?: ((e: { component?: dxTreeView, element?: TElement, model?: any }) => void);
    /**
     */
    parentIdExpr?: string | Function;
    /**
     */
    rootValue?: any;
    /**
     */
    scrollDirection?: 'both' | 'horizontal' | 'vertical';
    /**
     */
    selectAllText?: string;
    /**
     */
    selectByClick?: boolean;
    /**
     */
    selectNodesRecursive?: boolean;
    /**
     */
    selectionMode?: 'multiple' | 'single';
    /**
     */
    showCheckBoxesMode?: 'none' | 'normal' | 'selectAll';
    /**
     */
    virtualModeEnabled?: boolean;
}
/**
 */
export default class dxTreeView extends HierarchicalCollectionWidget {
    constructor(element: TElement, options?: dxTreeViewOptions)
    /**
     */
    collapseAll(): void;
    /**
     */
    collapseItem(itemData: any): TPromise<void>;
    /**
     */
    collapseItem(itemElement: Element): TPromise<void>;
    /**
     */
    collapseItem(key: any): TPromise<void>;
    /**
     */
    expandAll(): void;
    /**
     */
    expandItem(itemData: any): TPromise<void>;
    /**
     */
    expandItem(itemElement: Element): TPromise<void>;
    /**
     */
    expandItem(key: any): TPromise<void>;
    /**
     */
    getNodes(): Array<dxTreeViewNode>;
    /**
     */
    getSelectedNodes(): Array<dxTreeViewNode>;
    /**
     */
    getSelectedNodeKeys(): Array<any>;
    /**
     */
    selectAll(): void;
    /**
     */
    selectItem(itemData: any): boolean;
    /**
     */
    selectItem(itemElement: Element): boolean;
    /**
     */
    selectItem(key: any): boolean;
    /**
     */
    unselectAll(): void;
    /**
     */
    unselectItem(itemData: any): boolean;
    /**
     */
    unselectItem(itemElement: Element): boolean;
    /**
     */
    unselectItem(key: any): boolean;
    /**
     */
    updateDimensions(): TPromise<void>;
    /**
     */
    scrollToItem(itemData: any): TPromise<void>;
    /**
     */
    scrollToItem(itemElement: Element): TPromise<void>;
    /**
     */
    scrollToItem(key: any): TPromise<void>;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxTreeViewItem extends CollectionWidgetItem {
    /**
     */
    expanded?: boolean;
    /**
     */
    hasItems?: boolean;
    /**
     */
    icon?: string;
    /**
     */
    items?: Array<dxTreeViewItem>;
    /**
     */
    parentId?: number | string;
    /**
     */
    selected?: boolean;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxTreeViewNode {
    /**
     */
    children?: Array<dxTreeViewNode>;
    /**
     */
    disabled?: boolean;
    /**
     */
    expanded?: boolean;
    /**
     */
    itemData?: any;
    /**
     */
    key?: any;
    /**
     */
    parent?: dxTreeViewNode;
    /**
     */
    selected?: boolean;
    /**
     */
    text?: string;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxTreeViewOptions;

/** @deprecated use Options instead */
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxTreeViewOptions;
