/**
* DevExtreme (viz/bar_gauge.d.ts)
* Version: 21.1.0 (build 21089-0329)
* Build date: Tue Mar 30 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    PaletteType,
    PaletteExtensionModeType
} from './palette';

import {
    template
} from '../core/templates/template';

import {
    format
} from '../ui/widget/ui.widget';

import {
    BaseLegend,
    BaseLegendItem
} from './common';

import BaseWidget, {
    BaseWidgetLoadingIndicator,
    BaseWidgetOptions,
    BaseWidgetTooltip,
    Font
} from './core/base_widget';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BarGaugeBarInfo {
    /**
     */
    color?: string;
    /**
     */
    index?: number;
    /**
     */
    value?: number;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BarGaugeLegendItem extends BaseLegendItem {
    /**
     */
    item?: BarGaugeBarInfo;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxBarGaugeOptions extends BaseWidgetOptions<dxBarGauge> {
    /**
     */
    animation?: any;
    /**
     */
    backgroundColor?: string;
    /**
     */
    barSpacing?: number;
    /**
     */
    baseValue?: number;
    /**
     */
    endValue?: number;
    /**
     */
    geometry?: {
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      endAngle?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      startAngle?: number
    };
    /**
     */
    label?: {
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      connectorColor?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      connectorWidth?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      customizeText?: ((barValue: { value?: number, valueText?: string }) => string),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      font?: Font,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      format?: format,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      indent?: number,
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      visible?: boolean
    };
    /**
     */
    legend?: dxBarGaugeLegend;
    /**
     */
    loadingIndicator?: dxBarGaugeLoadingIndicator;
    /**
     */
    onTooltipHidden?: ((e: { component?: dxBarGauge, element?: TElement, model?: any, target?: any }) => void);
    /**
     */
    onTooltipShown?: ((e: { component?: dxBarGauge, element?: TElement, model?: any, target?: any }) => void);
    /**
     */
    palette?: Array<string> | PaletteType;
    /**
     */
    paletteExtensionMode?: PaletteExtensionModeType;
    /**
     */
    relativeInnerRadius?: number;
    /**
     */
    resolveLabelOverlapping?: 'hide' | 'none';
    /**
     */
    startValue?: number;
    /**
     */
    tooltip?: dxBarGaugeTooltip;
    /**
     */
    values?: Array<number>;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxBarGaugeLegend extends BaseLegend {
    /**
     * Specifies the hint that appears when a user hovers the mouse pointer over a legend item.
     */
    customizeHint?: ((arg: { item?: BarGaugeBarInfo, text?: string }) => string);
    /**
     * Allows you to change the order, text, and visibility of legend items.
     */
    customizeItems?: ((items: Array<BarGaugeLegendItem>) => Array<BarGaugeLegendItem>);
    /**
     * Customizes the text displayed by legend items.
     */
    customizeText?: ((arg: { item?: BarGaugeBarInfo, text?: string }) => string);
    /**
     * Formats the item text before it is displayed. Accepts only numeric formats. When unspecified, it inherits the label&apos;s format.
     */
    itemTextFormat?: format;
    /**
     * Specifies an SVG element that serves as a custom legend item marker.
     */
    markerTemplate?: template | ((legendItem: BarGaugeLegendItem, element: SVGGElement) => string | TElement<SVGElement>);
    /**
     * 
     */
    visible?: boolean;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxBarGaugeLoadingIndicator extends BaseWidgetLoadingIndicator {
    /**
     * 
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
    enabled?: boolean
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxBarGaugeTooltip extends BaseWidgetTooltip {
    /**
     * Specifies a custom template for a tooltip.
     */
    contentTemplate?: template | ((scaleValue: { value?: number, valueText?: string, index?: number }, element: TElement) => string | TElement);
    /**
     * Allows you to change tooltip appearance.
     */
    customizeTooltip?: ((scaleValue: { value?: number, valueText?: string, index?: number }) => any);
    /**
     * Allows users to interact with the tooltip content.
     */
    interactive?: boolean;
}
/**
 */
export default class dxBarGauge extends BaseWidget {
    constructor(element: TElement, options?: dxBarGaugeOptions)
    /**
     */
    values(): Array<number>;
    /**
     */
    values(values: Array<number>): void;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxBarGaugeOptions;

/** @deprecated use Options instead */
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxBarGaugeOptions;
