/**
* DevExtreme (viz/core/base_widget.d.ts)
* Version: 21.1.0 (build 21089-0329)
* Build date: Tue Mar 30 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import DOMComponent, {
    DOMComponentOptions
} from '../../core/dom_component';

import {
  Device
} from '../../core/devices';

import {
    TElement
} from '../../core/element';

import {
    TPromise
} from '../../core/utils/deferred';

import {
    format
} from '../../ui/widget/ui.widget';

import {
    DashStyleType
} from '../common';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type WordWrapType = 'normal' | 'breakWord' | 'none';
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type VizTextOverflowType = 'ellipsis' | 'hide' | 'none';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseWidgetOptions<T = BaseWidget> extends DOMComponentOptions<T> {
    /**
     */
    disabled?: boolean;
    /**
     */
    export?: BaseWidgetExport;
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
    height?: number | string | (() => number | string);
    /**
     */
    loadingIndicator?: BaseWidgetLoadingIndicator;
    /**
     */
    margin?: BaseWidgetMargin;
    /**
     */
    onDrawn?: ((e: { component?: T, element?: TElement, model?: any }) => void);
    /**
     */
    onExported?: ((e: { component?: T, element?: TElement, model?: any }) => void);
    /**
     */
    onExporting?: ((e: { component?: T, element?: TElement, model?: any, fileName?: string, cancel?: boolean, format?: string }) => void);
    /**
     */
    onFileSaving?: ((e: { component?: T, element?: TElement, fileName?: string, format?: string, data?: Blob, cancel?: boolean }) => void);
    /**
     */
    onIncidentOccurred?: ((e: { component?: T, element?: TElement, model?: any, target?: any }) => void);
    /**
     */
    pathModified?: boolean;
    /**
     */
    redrawOnResize?: boolean;
    /**
     */
    rtlEnabled?: boolean;
    /**
     */
    size?: BaseWidgetSize;
    /**
     */
    theme?: 'generic.dark' | 'generic.light' | 'generic.contrast' | 'generic.carmine' | 'generic.darkmoon' | 'generic.darkviolet' | 'generic.greenmist' | 'generic.softblue' | 'material.blue.light' | 'material.lime.light' | 'material.orange.light' | 'material.purple.light' | 'material.teal.light';
    /**
     */
    title?: BaseWidgetTitle | string;
    /**
     */
    tooltip?: BaseWidgetTooltip;
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
    width?: number | string | (() => number | string);
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseWidgetExport {
    /**
     * Specifies the color that will fill transparent regions in the resulting file or document.
     */
    backgroundColor?: string;
    /**
     * Enables the client-side exporting in the UI component.
     */
    enabled?: boolean;
    /**
     * Specifies a default name for the file to which the UI component will be exported.
     */
    fileName?: string;
    /**
     * Specifies a set of export formats.
     */
    formats?: Array<'GIF' | 'JPEG' | 'PDF' | 'PNG' | 'SVG'>;
    /**
     * Adds an empty space around the exported UI component; measured in pixels.
     */
    margin?: number;
    /**
     * Enables the printing feature in the UI component. Applies only if the export.enabled property is true.
     */
    printingEnabled?: boolean;
    /**
     * Specifies the URL of the server-side proxy that streams the resulting file to the end user to enable exporting in the Safari browser.
     * @deprecated Since v10, Safari browser supports API for saving files, and this property is no longer required.
     */
    proxyUrl?: string;
    /**
     * A function that renders SVG markup on the HTML canvas. Required to export custom SVG elements (for example, markerTemplate).
     */
    svgToCanvas?: ((svg: SVGElement, canvas: HTMLCanvasElement) => TPromise<void>);
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseWidgetLoadingIndicator {
    /**
     * Colors the background of the loading indicator.
     */
    backgroundColor?: string;
    /**
     * Specifies whether the loading indicator should be displayed and hidden automatically.
     */
    enabled?: boolean;
    /**
     * Specifies font properties for the loading indicator.
     */
    font?: Font;
    /**
     * Allows you to change the loading indicator&apos;s visibility.
     */
    show?: boolean;
    /**
     * Specifies the text to be displayed by the loading indicator.
     */
    text?: string;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseWidgetMargin {
    /**
     * Specifies the bottom margin of the UI component in pixels.
     */
    bottom?: number;
    /**
     * Specifies the left margin of the UI component in pixels.
     */
    left?: number;
    /**
     * Specifies the right margin of the UI component in pixels.
     */
    right?: number;
    /**
     * Specifies the top margin of the UI component in pixels.
     */
    top?: number;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseWidgetSize {
    /**
     * Specifies the height of the UI component in pixels.
     */
    height?: number;
    /**
     * Specifies the width of the UI component in pixels.
     */
    width?: number;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseWidgetTitle {
    /**
     * Specifies font properties for the title.
     */
    font?: Font;
    /**
     * Specifies the title&apos;s alignment in a horizontal direction.
     */
    horizontalAlignment?: 'center' | 'left' | 'right';
    /**
     * Generates space around the title.
     */
    margin?: number | {
      /**
       * Specifies the bottom margin of the title.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      bottom?: number,
      /**
       * Specifies the left margin of the title.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      left?: number,
      /**
       * Specifies the right margin of the title.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      right?: number,
      /**
       * Specifies the top margin of the title.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      top?: number
    };
    /**
     * Reserves a pixel-measured space for the title.
     */
    placeholderSize?: number;
    /**
     * Configures the UI component&apos;s subtitle.
     */
    subtitle?: {
      /**
       * Specifies font properties for the subtitle.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      font?: Font,
      /**
       * Specifies the distance between the title and subtitle in pixels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      offset?: number,
      /**
       * Specifies text for the subtitle.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      text?: string,
      /**
       * Specifies what to do with the subtitle when it overflows the allocated space after applying wordWrap: hide, truncate it and display an ellipsis, or do nothing.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      textOverflow?: VizTextOverflowType,
      /**
       * Specifies how to wrap the subtitle if it does not fit into a single line.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      wordWrap?: WordWrapType
    } | string;
    /**
     * Specifies the title&apos;s text.
     */
    text?: string;
    /**
     * Specifies what to do with the title when it overflows the allocated space after applying wordWrap: hide, truncate it and display an ellipsis, or do nothing.
     */
    textOverflow?: VizTextOverflowType;
    /**
     * Specifies the title&apos;s alignment in a vertical direction.
     */
    verticalAlignment?: 'bottom' | 'top';
    /**
     * Specifies how to wrap the title if it does not fit into a single line.
     */
    wordWrap?: WordWrapType;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseWidgetTooltip {
    /**
     * Specifies the length of a tooltip&apos;s arrow in pixels.
     */
    arrowLength?: number;
    /**
     * Configures a tooltip&apos;s border.
     */
    border?: {
      /**
       * Colors a tooltip&apos;s border.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
      /**
       * Specifies the dash style of a tooltip&apos;s border.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      dashStyle?: DashStyleType,
      /**
       * Specifies the transparency of a tooltip&apos;s border.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      opacity?: number,
      /**
       * Specifies whether a tooltip&apos;s border is visible.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean,
      /**
       * Specifies the width of a tooltip&apos;s border in pixels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     * Colors all tooltips.
     */
    color?: string;
    /**
     * Specifies the container in which to draw tooltips. The default container is the HTML DOM `` element.
     */
    container?: string | TElement;
    /**
     * Makes all the tooltip&apos;s corners rounded.
     */
    cornerRadius?: number;
    /**
     * Enables tooltips.
     */
    enabled?: boolean;
    /**
     * Specifies tooltips&apos; font properties.
     */
    font?: Font;
    /**
     * Formats a value before it is displayed it in a tooltip.
     */
    format?: format;
    /**
     * Specifies tooltips&apos; transparency.
     */
    opacity?: number;
    /**
     * Generates an empty space, measured in pixels, between a tooltip&apos;s left/right border and its text.
     */
    paddingLeftRight?: number;
    /**
     * Generates an empty space, measured in pixels, between a tooltip&apos;s top/bottom border and its text.
     */
    paddingTopBottom?: number;
    /**
     * Configures a tooltip&apos;s shadow.
     */
    shadow?: {
      /**
       * Specifies the blur distance of a tooltip&apos;s shadow. The larger the value, the blurrier the shadow&apos;s edge.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      blur?: number,
      /**
       * Colors a tooltip&apos;s shadow.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
      /**
       * Specifies the horizontal offset of a tooltip&apos;s shadow relative to the tooltip itself. Measured in pixels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      offsetX?: number,
      /**
       * Specifies the vertical offset of a tooltip&apos;s shadow relative to the tooltip itself. Measured in pixels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      offsetY?: number,
      /**
       * Specifies the transparency of a tooltip&apos;s shadow.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      opacity?: number
    };
    /**
     * Specifies a tooltip&apos;s z-index.
     */
    zIndex?: number;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export default class BaseWidget extends DOMComponent {
    constructor(element: TElement, options?: BaseWidgetOptions)
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
    static defaultOptions(rule: { device?: Device | Array<Device> | Function, options?: any }): void;
    /**
     */
    exportTo(fileName: string, format: string): void;
    /**
     */
    getSize(): BaseWidgetSize;
    /**
     */
    hideLoadingIndicator(): void;
    /**
     */
    print(): void;
    /**
     */
    render(): void;
    /**
     */
    showLoadingIndicator(): void;
    /**
     */
    svg(): string;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface Font {
    /**
     */
    color?: string;
    /**
     */
    family?: string;
    /**
     */
    opacity?: number;
    /**
     */
    size?: string | number;
    /**
     */
    weight?: number;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseWidgetAnnotationConfig {
    /**
     */
    allowDragging?: boolean;
    /**
     */
    arrowLength?: number;
    /**
     */
    arrowWidth?: number;
    /**
     */
    border?: {
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      cornerRadius?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      dashStyle?: DashStyleType,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      opacity?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     */
    color?: string;
    /**
     */
    data?: any;
    /**
     */
    description?: string;
    /**
     */
    font?: Font;
    /**
     */
    height?: number;
    /**
     */
    image?: string | {
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      height?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      url?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     */
    offsetX?: number;
    /**
     */
    offsetY?: number;
    /**
     */
    opacity?: number;
    /**
     */
    paddingLeftRight?: number;
    /**
     */
    paddingTopBottom?: number;
    /**
     */
    shadow?: {
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      blur?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      offsetX?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      offsetY?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      opacity?: number
    };
    /**
     */
    text?: string;
    /**
     */
    textOverflow?: VizTextOverflowType;
    /**
     */
    tooltipEnabled?: boolean;
    /**
     */
    type?: 'text' | 'image' | 'custom';
    /**
     */
    width?: number;
    /**
     */
    wordWrap?: WordWrapType;
    /**
     */
    x?: number;
    /**
     */
    y?: number;
}
