/**
* DevExtreme (viz/gauges/base_gauge.d.ts)
* Version: 21.1.0 (build 21089-0329)
* Build date: Tue Mar 30 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../../core/element';

import {
    PaletteType,
    PaletteExtensionModeType
} from '../palette';

import {
    template
} from '../../core/templates/template';

import {
    format
} from '../../ui/widget/ui.widget';

import BaseWidget, {
    BaseWidgetLoadingIndicator,
    BaseWidgetOptions,
    BaseWidgetTooltip,
    Font
} from '../core/base_widget';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseGaugeOptions<T = BaseGauge> extends BaseWidgetOptions<T> {
    /**
     */
    animation?: BaseGaugeAnimation;
    /**
     */
    containerBackgroundColor?: string;
    /**
     */
    loadingIndicator?: BaseGaugeLoadingIndicator;
    /**
     */
    onTooltipHidden?: ((e: { component?: T, element?: TElement, model?: any, target?: any }) => void);
    /**
     */
    onTooltipShown?: ((e: { component?: T, element?: TElement, model?: any, target?: any }) => void);
    /**
     */
    rangeContainer?: BaseGaugeRangeContainer;
    /**
     */
    scale?: BaseGaugeScale;
    /**
     */
    subvalues?: Array<number>;
    /**
     */
    tooltip?: BaseGaugeTooltip;
    /**
     */
    value?: number;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseGaugeAnimation {
    /**
     * Determines how long animation runs.
     */
    duration?: number;
    /**
     * Specifies the animation easing mode.
     */
    easing?: 'easeOutCubic' | 'linear';
    /**
     * Indicates whether or not animation is enabled.
     */
    enabled?: boolean;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseGaugeLoadingIndicator extends BaseWidgetLoadingIndicator {
    /**
     * 
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
    enabled?: boolean;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseGaugeRangeContainer {
    /**
     * Specifies a range container&apos;s background color.
     */
    backgroundColor?: string;
    /**
     * Specifies the offset of the range container from an invisible scale line in pixels.
     */
    offset?: number;
    /**
     * Specifies the palette to be used for colorizing ranges in the range container.
     */
    palette?: Array<string> | PaletteType;
    /**
     * Specifies what to do with colors in the palette when their number is less than the number of ranges in the range container.
     */
    paletteExtensionMode?: PaletteExtensionModeType;
    /**
     * An array of objects representing ranges contained in the range container.
     */
    ranges?: Array<{
      /**
       * Specifies a color of a range.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
      /**
       * Specifies an end value of a range.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      endValue?: number,
      /**
       * Specifies a start value of a range.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      startValue?: number
    }>;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseGaugeScale {
    /**
     * Specifies whether to allow decimal values on the scale. When false, the scale contains integer values only.
     */
    allowDecimals?: boolean;
    /**
     * Specifies an array of custom minor ticks.
     */
    customMinorTicks?: Array<number>;
    /**
     * Specifies an array of custom major ticks.
     */
    customTicks?: Array<number>;
    /**
     * Specifies the end value for the scale of the gauge.
     */
    endValue?: number;
    /**
     * Specifies common properties for scale labels.
     */
    label?: BaseGaugeScaleLabel;
    /**
     * Specifies properties of the gauge&apos;s minor ticks.
     */
    minorTick?: {
      /**
       * Specifies the color of the scale&apos;s minor ticks.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
    /**
     * Specifies the length of the scale&apos;s minor ticks.
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      length?: number,
      /**
       * Specifies the opacity of the scale&apos;s minor ticks.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      opacity?: number,
      /**
       * Indicates whether scale minor ticks are visible or not.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean,
      /**
       * Specifies the width of the scale&apos;s minor ticks.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     * Specifies an interval between minor ticks.
     */
    minorTickInterval?: number;
    /**
     * Specifies the minimum distance between two neighboring major ticks in pixels.
     */
    scaleDivisionFactor?: number;
    /**
     * Specifies the start value for the scale of the gauge.
     */
    startValue?: number;
    /**
     * Specifies properties of the gauge&apos;s major ticks.
     */
    tick?: {
    /**
     * Specifies the color of the scale&apos;s major ticks.
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      color?: string,
      /**
       * Specifies the length of the scale&apos;s major ticks.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      length?: number,
      /**
       * Specifies the opacity of the scale&apos;s major ticks.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      opacity?: number,
      /**
       * Indicates whether scale major ticks are visible or not.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean,
      /**
       * Specifies the width of the scale&apos;s major ticks.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     * Specifies an interval between major ticks.
     */
    tickInterval?: number;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseGaugeScaleLabel {
    /**
     * Specifies a callback function that returns the text to be displayed in scale labels.
     */
    customizeText?: ((scaleValue: { value?: number, valueText?: string }) => string);
    /**
     * Specifies font properties for the text displayed in the scale labels of the gauge.
     */
    font?: Font;
    /**
     * Formats a value before it is displayed in a scale label. Accepts only numeric formats.
     */
    format?: format;
    /**
     * Decides how to arrange scale labels when there is not enough space to keep all of them.
     */
    overlappingBehavior?: 'hide' | 'none';
    /**
     * Specifies whether or not scale labels should be colored similarly to their corresponding ranges in the range container.
     */
    useRangeColors?: boolean;
    /**
     * Specifies whether or not scale labels are visible on the gauge.
     */
    visible?: boolean;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface BaseGaugeTooltip extends BaseWidgetTooltip {
    /**
     * Specifies a custom template for a tooltip.
     */
    contentTemplate?: template | ((scaleValue: { value?: number, valueText?: string }, element: TElement) => string | TElement);
    /**
     * Allows you to change the appearance of specified tooltips.
     */
    customizeTooltip?: ((scaleValue: { value?: number, valueText?: string }) => any);
    /**
     * Allows users to interact with the tooltip content.
     */
    interactive?: boolean;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export class BaseGauge extends BaseWidget {
    constructor(element: TElement, options?: BaseGaugeOptions)
    /**
     */
    subvalues(): Array<number>;
    /**
     */
    subvalues(subvalues: Array<number>): void;
    /**
     */
    value(): number;
    /**
     */
    value(value: number): void;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface CommonIndicator {
    /**
     */
    arrowLength?: number;
    /**
     */
    backgroundColor?: string;
    /**
     */
    baseValue?: number;
    /**
     */
    beginAdaptingAtRadius?: number;
    /**
     */
    color?: string;
    /**
     */
    horizontalOrientation?: 'left' | 'right';
    /**
     */
    indentFromCenter?: number;
    /**
     */
    length?: number;
    /**
     */
    offset?: number;
    /**
     */
    palette?: Array<string> | PaletteType;
    /**
     */
    secondColor?: string;
    /**
     */
    secondFraction?: number;
    /**
     */
    size?: number;
    /**
     */
    spindleGapSize?: number;
    /**
     */
    spindleSize?: number;
    /**
     */
    text?: {
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      customizeText?: ((indicatedValue: { value?: number, valueText?: string }) => string),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      font?: Font,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      format?: format,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      indent?: number
    };
    /**
     */
    verticalOrientation?: 'bottom' | 'top';
    /**
     */
    width?: number;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type GaugeIndicatorType = 'circle' | 'rangeBar' | 'rectangle' | 'rectangleNeedle' | 'rhombus' | 'textCloud' | 'triangleMarker' | 'triangleNeedle' | 'twoColorNeedle';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface GaugeIndicator extends CommonIndicator {
    /**
     */
    type?: GaugeIndicatorType;
}
